# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest
import time
import os

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer

TEST_DIR = os.path.dirname(os.path.realpath(__file__))


class ResourceGroupScenarioTest(ScenarioTest):

    def test_rest_arm(self):
        from knack.util import CLIError

        self.kwargs.update({
            'rg': self.create_random_name("test-rest-rg", length=20)
        })

        # Test ARM Subscriptions - List
        # https://docs.microsoft.com/en-us/rest/api/resources/subscriptions/list
        self.cmd('az rest -u /subscriptions?api-version=2020-01-01',
                 checks=[self.exists("value")])

        # Test ARM Tenants - List
        # https://docs.microsoft.com/en-us/rest/api/resources/tenants/list
        self.cmd('az rest -u /tenants?api-version=2020-01-01',
                 checks=[self.exists("value")])

        # Resource Groups - Create Or Update
        # https://docs.microsoft.com/en-us/rest/api/resources/resourcegroups/createorupdate
        self.cmd('az rest -m PUT -u https://management.azure.com/subscriptions/{{subscriptionId}}/resourcegroups/{rg}?api-version=2019-10-01 '
                 '--body \'{{"location": "eastus"}}\'',
                 checks=[self.check("name", '{rg}')])

        # Resource Groups - Get
        # https://docs.microsoft.com/en-us/rest/api/resources/resourcegroups/get
        self.cmd('az rest -u https://management.azure.com/subscriptions/{{subscriptionId}}/resourcegroups/{rg}?api-version=2019-10-01',
                 checks=[self.check("name", '{rg}')])

        # Resource Groups - Delete
        # https://docs.microsoft.com/en-us/rest/api/resources/resourcegroups/delete
        self.cmd('az rest -m DELETE -u https://management.azure.com/subscriptions/{{subscriptionId}}/resourcegroups/{rg}?api-version=2019-10-01',
                 checks=[])

        # Resource Groups - Get
        # Polling for 404
        # TODO: return 3 for 404
        with self.assertRaises(CLIError):
            while True:
                time.sleep(5)
                self.cmd('az rest -u https://management.azure.com/subscriptions/{{subscriptionId}}/resourcegroups/{rg}?api-version=2019-10-01',
                         checks=[])

    @ResourceGroupPreparer(name_prefix='cli_test_rest')
    def test_rest_storage(self, resource_group):

        self.kwargs.update({
            'sa': self.create_random_name("tmpst", length=10),
            # Please remember to add double quote to the variable reference of the file path in tests,
            # otherwise \\ will be parsed to \, which will lead to the wrong path
            'storage_account_create_body': os.path.join(TEST_DIR, 'rest_storage_account_create_body.json'),
            'storage_account_sas_body': os.path.join(TEST_DIR, 'rest_storage_account_sas_body.json')
        })

        # Create a storage account
        # https://docs.microsoft.com/en-us/rest/api/storagerp/storageaccounts/create
        self.cmd('az rest -m PUT -u /subscriptions/{{subscriptionId}}/resourceGroups/{rg}/providers/Microsoft.Storage/storageAccounts/{sa}?api-version=2019-06-01 '
                 '-b @"{storage_account_create_body}"')

        # Poll on the provisioning state
        state = None
        while state != 'Succeeded':
            time.sleep(5)
            # Show the storage account
            # https://docs.microsoft.com/en-us/rest/api/storagerp/storageaccounts/getproperties
            state = self.cmd('az rest -m GET -u /subscriptions/{{subscriptionId}}/resourceGroups/{rg}/providers/Microsoft.Storage/storageAccounts/{sa}?api-version=2019-06-01') \
                .get_output_in_json()["properties"]["provisioningState"]

        # Create an account SAS token https://docs.microsoft.com/en-us/rest/api/storageservices/create-account-sas
        # https://docs.microsoft.com/en-us/rest/api/storagerp/storageaccounts/listaccountsas
        sas_token = self.cmd('az rest -m POST -u "/subscriptions/{{subscriptionId}}/resourceGroups/{rg}/providers/Microsoft.Storage/storageAccounts/{sa}/ListAccountSas?api-version=2019-06-01" '
                             '-b @"{storage_account_sas_body}"').get_output_in_json()["accountSasToken"]

        # Create a container with the SAS token
        # https://docs.microsoft.com/en-us/rest/api/storageservices/create-container
        self.cmd('az rest -m PUT -u https://{sa}.blob.core.windows.net/mycontainer?restype=container&' + sas_token + ' '
                 '--skip-authorization-header')

        # Show the container properties
        # https://docs.microsoft.com/en-us/rest/api/storageservices/get-container-properties
        self.cmd('az rest -m HEAD -u https://{sa}.blob.core.windows.net/mycontainer?restype=container&' + sas_token + ' '
                 '--skip-authorization-header')

        # Create a blob
        # https://docs.microsoft.com/en-us/rest/api/storageservices/put-blob
        self.cmd('az rest -m PUT -u https://{sa}.blob.core.windows.net/mycontainer/myblob?' + sas_token + ' '
                 '--headers "Content-Type=text/plain; charset=UTF-8" "x-ms-blob-type=BlockBlob" '
                 '--skip-authorization-header '
                 '--body "hello world"')

        # Show the blob
        # https://docs.microsoft.com/en-us/rest/api/storageservices/get-blob
        self.cmd('az rest -m GET -u https://{sa}.blob.core.windows.net/mycontainer/myblob?' + sas_token + ' '
                 '--skip-authorization-header')

        # List blobs in the container
        # https://docs.microsoft.com/en-us/rest/api/storageservices/list-blobs
        self.cmd('az rest -m GET -u https://{sa}.blob.core.windows.net/mycontainer?restype=container&comp=list&' + sas_token + ' '
                 '--skip-authorization-header')

        # Delete the blob
        # https://docs.microsoft.com/en-us/rest/api/storageservices/delete-blob
        self.cmd('az rest -m DELETE -u https://{sa}.blob.core.windows.net/mycontainer/myblob?' + sas_token + ' '
                 '--skip-authorization-header')

        # Delete the container
        # https://docs.microsoft.com/en-us/rest/api/storageservices/delete-container
        self.cmd('az rest -m DELETE -u https://{sa}.blob.core.windows.net/mycontainer?restype=container&' + sas_token + ' '
                 '--skip-authorization-header')

        # Delete the storage account
        # https://docs.microsoft.com/en-us/rest/api/storagerp/storageaccounts/delete
        self.cmd('az rest -m DELETE -u "/subscriptions/{{subscriptionId}}/resourceGroups/{rg}/providers/Microsoft.Storage/storageAccounts/{sa}?api-version=2019-06-01"')

    def test_rest_microsoft_graph(self):
        from knack.util import CLIError

        display_name_prefix = "az-rest-test-app"
        self.kwargs.update({
            'display_name_prefix': display_name_prefix,
            'display_name': self.create_random_name(display_name_prefix, length=25)
        })

        # Create application
        # https://docs.microsoft.com/en-us/graph/api/application-post-applications?view=graph-rest-1.0&tabs=http
        # Escape single quotes for `shlex` and curly braces for `format`
        app = self.cmd('az rest --method POST --url https://graph.microsoft.com/v1.0/applications --body \'{{"displayName": "{display_name}"}}\'',
                       checks=[
                           self.check('displayName', '{display_name}')
                       ]).get_output_in_json()

        self.kwargs["app_object_id"] = app["id"]
        self.kwargs["app_id"] = app["appId"]

        # Get application
        # https://docs.microsoft.com/en-us/graph/api/application-get?view=graph-rest-1.0&tabs=http
        self.cmd('az rest --method GET --url https://graph.microsoft.com/v1.0/applications/{app_object_id}',
                 checks=[self.check('displayName', '{display_name}')])

        # Update application
        # https://docs.microsoft.com/en-us/graph/api/application-update?view=graph-rest-1.0&tabs=http
        self.cmd('az rest --method PATCH --url https://graph.microsoft.com/v1.0/applications/{app_object_id} --body \'{{"web":{{"redirectUris":["https://myapp.com"]}}}}\'')

        # application: addPassword
        # https://docs.microsoft.com/en-us/graph/api/application-addpassword?view=graph-rest-1.0&tabs=http
        self.cmd('az rest --method POST --url https://graph.microsoft.com/v1.0/applications/{app_object_id}/addPassword '
                 '--body \'{{"passwordCredential": {{"displayName": "Password friendly name"}}}}\'',
                 checks=[self.check('displayName', "Password friendly name")])

        # Create servicePrincipal
        # https://docs.microsoft.com/en-us/graph/api/serviceprincipal-post-serviceprincipals?view=graph-rest-1.0&tabs=http
        sp = self.cmd('az rest --method POST --url https://graph.microsoft.com/v1.0/serviceprincipals --body \'{{"appId": "{app_id}"}}\'').get_output_in_json()
        self.kwargs["sp_object_id"] = sp["id"]

        # Get servicePrincipal
        # https://docs.microsoft.com/en-us/graph/api/serviceprincipal-get?view=graph-rest-1.0&tabs=http
        self.cmd('az rest --method GET --url https://graph.microsoft.com/v1.0/servicePrincipals/{sp_object_id}',
                 checks=[self.check('appId', '{app_id}'),
                         self.check('id', '{sp_object_id}')])

        # Update servicePrincipal
        # https://docs.microsoft.com/en-us/graph/api/serviceprincipal-update?view=graph-rest-1.0&tabs=http
        self.cmd('az rest --method PATCH --url https://graph.microsoft.com/v1.0/servicePrincipals/{sp_object_id} '
                 '--body \'{{"appRoleAssignmentRequired": true}}\'')

        # servicePrincipal: addPassword
        # https://docs.microsoft.com/en-us/graph/api/serviceprincipal-addpassword?view=graph-rest-1.0&tabs=http
        self.cmd('az rest --method POST --url https://graph.microsoft.com/v1.0/servicePrincipals/{sp_object_id}/addPassword '
                 '--body \'{{"passwordCredential": {{"displayName": "Password friendly name"}}}}\'',
                 checks=[self.check('displayName', "Password friendly name")])

        # Delete servicePrincipal
        # https://docs.microsoft.com/en-us/graph/api/serviceprincipal-delete?view=graph-rest-1.0&tabs=http
        self.cmd('az rest --method DELETE --url https://graph.microsoft.com/v1.0/serviceprincipals/{sp_object_id}')

        with self.assertRaisesRegex(CLIError, "Request_ResourceNotFound"):
            self.cmd('az rest --method GET --url https://graph.microsoft.com/v1.0/servicePrincipals/{sp_object_id}')

        # Delete application
        # https://docs.microsoft.com/en-us/graph/api/application-delete?view=graph-rest-1.0&tabs=http
        self.cmd('az rest --method DELETE --url https://graph.microsoft.com/v1.0/applications/{app_object_id}')

        with self.assertRaisesRegex(CLIError, "Request_ResourceNotFound"):
            self.cmd('az rest --method GET --url https://graph.microsoft.com/v1.0/applications/{app_object_id}')

        # Clear the trash left behind by failed tests
        response = self.cmd('az rest -m GET -u "https://graph.microsoft.com/v1.0/applications?$filter=startswith(displayName, \'{display_name_prefix}\')"').get_output_in_json()
        apps = response['value']
        for app in apps:
            self.cmd('az rest --method DELETE --url https://graph.microsoft.com/v1.0/applications/{}'.format(app['id']))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import xmltodict
import subprocess
import logging
import re
import sys
import os

logger = logging.getLogger('az-publish')

pattern = r".*?\s+(( --\S+)+( -\S)*).*?:"
# meaning:  escape ansi color codes then
#           match one or more long options ('--') followed by an (optional) short option ('-')
#           and then the first semicolon after 0 or more chars (':')
#           Also handles cases where there are multiple long options see --edition in az sql mi
COMPILED_RE = re.compile(pattern)
GLOBAL_ARGS = ["debug", "help", "output", "query", "subscription", "verbose"]


def main(xml_file, commands=[]):

    if commands is None:
        commands = ["vm create"]
    with open(xml_file, 'r') as f:
        doc_contents = xmltodict.parse(f.read(), process_namespaces=True)

    entries = doc_contents['document']['desc']
    cmd_to_content_dict = {entry['desc_signature']['desc_addname']['#text']: entry['desc_content'] for entry in
                           entries if entry['@objtype'] == "clicommand"}
    for cmd in commands:
        logger.warning("Testing %s help...", cmd)
        help_args = get_args_from_help_text(cmd)
        doc_args = get_args_from_doc_dict(cmd, cmd_to_content_dict)
        try:
            assert help_args == doc_args
        except AssertionError as e:
            logger.error("Test failed. Help args are different.\n")
            logger.error("CLI help args:%s\n\nDoc help args: %s.\n", ", ".join(help_args), ", ".join(doc_args))
            logger.error(e)
            exit(1)


    logger.info("Doc help arguments match CLI help arguments.")



def get_args_from_help_text(cmd_name):
    cmd = "az {} -h".format(cmd_name)
    cmd_args = cmd.split()
    completedprocess = subprocess.run(cmd_args, stdout=subprocess.PIPE, encoding="utf-8")

    if completedprocess.returncode != 0:
        logger.error("Error running %s", " ".join(cmd))
        exit(1)

    lines = completedprocess.stdout.splitlines()
    arguments = set()
    for line in lines:
        match = COMPILED_RE.search(line)
        if match:
            options = match.group(1).strip()
            if _get_long_option(options) not in GLOBAL_ARGS:
                arguments.add(options)
    return arguments


def get_args_from_doc_dict(cmd_name, cmd_to_content_dict):
    try:
        arg_info_dict = {possible_arg['desc_signature']['desc_addname']['#text']: possible_arg['desc_content']['field_list']['field']
                for possible_arg in cmd_to_content_dict[cmd_name]['desc'] if possible_arg['@objtype'] == "cliarg" and
                _get_long_option(possible_arg['desc_signature']['desc_addname']['#text']) not in GLOBAL_ARGS}
    except KeyError:
        return set()

    arguments = set()
    # Only add arguments that don't have the deprecated field. #todo: tidy up logic.
    for arg, arg_fields in arg_info_dict.items():
        is_deprecated = False
        for field in arg_fields:
            if field['field_name'].lower() == 'deprecated':
                is_deprecated = True
                break
        if not is_deprecated:
            arguments.add(arg)
    return arguments



def _get_long_option(options):
    return options.strip().split()[0].lstrip("-")


if __name__ == "__main__":
    usage_msg = "USAGE: PATH-TO-DOC-XML [optional command (without az)]"

    # very rudimentary arg parsing TODO: update to argparse so list of commands can be passed in.
    if len(sys.argv) < 2:
        logger.warning(usage_msg)
        exit(1)

    if not os.path.isfile(sys.argv[1]):
        logger.warning(usage_msg)
        exit(1)

    default_commands = [
        "vm", "network vnet", "storage", # these groups should not fail as command groups do not have parameters.
       "keyvault secret set", "vm create", "sql mi create", "aks create", "webapp deployment slot list"]

    test_commands = [sys.argv[2]] if len(sys.argv) > 2 else default_commands
    main(sys.argv[1], test_commands)


# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import unittest

from azure.mgmt.containerservice.v2020_09_01.models import ManagedClusterLoadBalancerProfile
from azure.mgmt.containerservice.v2020_09_01.models import ManagedClusterLoadBalancerProfileManagedOutboundIPs
from azure.mgmt.containerservice.v2020_09_01.models import ManagedClusterLoadBalancerProfileOutboundIPPrefixes
from azure.mgmt.containerservice.v2020_09_01.models import ManagedClusterLoadBalancerProfileOutboundIPs
from azure.cli.core.util import CLIError
from azure.cli.command_modules.acs import _loadbalancer as loadbalancer


class TestLoadBalancer(unittest.TestCase):
    def test_configure_load_balancer_profile(self):
        managed_outbound_ip_count = 5
        outbound_ips = None
        outbound_ip_prefixes = None
        outbound_ports = 80
        idle_timeout = 3600

        profile = ManagedClusterLoadBalancerProfile()
        profile.managed_outbound_ips = ManagedClusterLoadBalancerProfileManagedOutboundIPs(
            count=2
        )
        profile.outbound_ips = ManagedClusterLoadBalancerProfileOutboundIPs(
            public_ips="public_ips"
        )
        profile.outbound_ip_prefixes = ManagedClusterLoadBalancerProfileOutboundIPPrefixes(
            public_ip_prefixes="public_ip_prefixes"
        )

        p = loadbalancer.configure_load_balancer_profile(managed_outbound_ip_count, outbound_ips, outbound_ip_prefixes, outbound_ports, idle_timeout, profile)

        self.assertIsNotNone(p.managed_outbound_ips)
        self.assertIsNone(p.outbound_ips)
        self.assertIsNone(p.outbound_ip_prefixes)


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os

from azure.cli.core.util import CLIError
from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer
from azure.cli.command_modules.ams._test_utils import _get_test_data_file


class AmsLiveEventTests(ScenarioTest):
    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_live_event_create(self, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)
        live_event_name = self.create_random_name(prefix='le', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'centralus',
            'streamingProtocol': 'RTMP',
            'liveEventName': live_event_name,
            'encodingType': 'Basic',
            'tags': 'key=value',
            'previewLocator': self.create_guid(),
            'keyFrameIntervalDuration': 'PT2S',
            'description': 'asd',
            'accessToken': '0abf356884d74b4aacbd7b1ebd3da0f7',
            'clientAccessPolicy': '@' + _get_test_data_file('clientAccessPolicy.xml'),
            'crossDomainPolicy': '@' + _get_test_data_file('crossDomainPolicy.xml')
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}', checks=[
            self.check('name', '{amsname}'),
            self.check('location', 'Central US')
        ])

        live_event = self.cmd('az ams live-event create -a {amsname} -n {liveEventName} -g {rg} --auto-start --streaming-protocol {streamingProtocol} --encoding-type {encodingType} --tags {tags} --stream-options Default LowLatency --preview-locator {previewLocator} --ips 1.2.3.4 5.6.7.8 192.168.0.0/28 --preview-ips 192.168.0.0/28 0.0.0.0 --key-frame-interval-duration {keyFrameIntervalDuration} --access-token {accessToken} --description {description} --client-access-policy "{clientAccessPolicy}" --cross-domain-policy "{crossDomainPolicy}" --vanity-url', checks=[
            self.check('name', '{liveEventName}'),
            self.check('location', 'Central US'),
            self.check('input.streamingProtocol', '{streamingProtocol}'),
            self.check('encoding.encodingType', '{encodingType}'),
            self.check('length(preview.accessControl.ip.allow)', 2),
            self.check('length(input.accessControl.ip.allow)', 3),
            self.check('preview.previewLocator', '{previewLocator}'),
            self.check('input.keyFrameIntervalDuration', '{keyFrameIntervalDuration}'),
            self.check('length(streamOptions)', 2),
            self.check('description', '{description}'),
            self.check('input.accessToken', '{accessToken}'),
            self.check('vanityUrl', True),
            self.check('input.accessControl.ip.allow[2].address', '192.168.0.0'),
            self.check('input.accessControl.ip.allow[2].subnetPrefixLength', '28'),
            self.check('preview.accessControl.ip.allow[0].address', '192.168.0.0'),
            self.check('preview.accessControl.ip.allow[0].subnetPrefixLength', '28')
        ]).get_output_in_json()

        self.assertIsNotNone(live_event['crossSiteAccessPolicies']['crossDomainPolicy'])
        self.assertIsNotNone(live_event['crossSiteAccessPolicies']['clientAccessPolicy'])

        self.assertNotEquals('Stopping', live_event['resourceState'])
        self.assertNotEquals('Stopped', live_event['resourceState'])

        self.cmd('az ams live-event stop -a {amsname} -n {liveEventName} -g {rg}')

        self.cmd('az ams live-event delete -a {amsname} -n {liveEventName} -g {rg}')

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_live_event_start(self, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)
        live_event_name = self.create_random_name(prefix='le', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'southindia',
            'streamingProtocol': 'FragmentedMP4',
            'liveEventName': live_event_name,
            'encodingType': 'Basic'
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}', checks=[
            self.check('name', '{amsname}'),
            self.check('location', 'South India')
        ])

        self.cmd('az ams live-event create -a {amsname} -n {liveEventName} -g {rg} --streaming-protocol {streamingProtocol} --encoding-type {encodingType} --tags key=value --ips AllowAll', checks=[
            self.check('name', '{liveEventName}'),
            self.check('location', 'South India'),
            self.check('input.streamingProtocol', 'FragmentedMP4'),
            self.check('encoding.encodingType', 'Basic')
        ])

        live_event = self.cmd('az ams live-event start -a {amsname} --name {liveEventName} -g {rg}', checks=[
            self.check('name', '{liveEventName}'),
            self.check('location', 'South India'),
            self.check('input.streamingProtocol', 'FragmentedMP4')
        ]).get_output_in_json()

        self.assertNotEquals('Stopping', live_event['resourceState'])
        self.assertNotEquals('Stopped', live_event['resourceState'])

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_live_event_stop(self, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)
        live_event_name = self.create_random_name(prefix='le', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'brazilsouth',
            'streamingProtocol': 'FragmentedMP4',
            'liveEventName': live_event_name,
            'encodingType': 'Basic'
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}', checks=[
            self.check('name', '{amsname}'),
            self.check('location', 'Brazil South')
        ])

        self.cmd('az ams live-event create -a {amsname} -n {liveEventName} -g {rg} --streaming-protocol {streamingProtocol} --encoding-type {encodingType} --tags key=value --ips AllowAll --auto-start')

        live_event = self.cmd('az ams live-event stop -a {amsname} -n {liveEventName} -g {rg}', checks=[
            self.check('name', '{liveEventName}')
        ]).get_output_in_json()

        self.assertNotEquals('Starting', live_event['resourceState'])
        self.assertNotEquals('Running', live_event['resourceState'])

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_live_event_stop_and_remove_outputs(self, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)
        live_event_name = self.create_random_name(prefix='le', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'japaneast',
            'streamingProtocol': 'FragmentedMP4',
            'liveEventName': live_event_name,
            'encodingType': 'Basic'
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}', checks=[
            self.check('name', '{amsname}'),
            self.check('location', 'Japan East')
        ])

        self.cmd('az ams live-event create -a {amsname} -n {liveEventName} -g {rg} --streaming-protocol {streamingProtocol} --encoding-type {encodingType} --tags key=value --ips AllowAll --auto-start')

        assetName = self.create_random_name(prefix='asset', length=12)
        live_output_name1 = self.create_random_name(prefix='lo1', length=12)
        live_output_name2 = self.create_random_name(prefix='lo2', length=12)
        manifest_name1 = self.create_random_name(prefix='man1', length=12)
        manifest_name2 = self.create_random_name(prefix='man2', length=12)

        self.kwargs.update({
            'assetName': assetName,
            'liveOutputName1': live_output_name1,
            'liveOutputName2': live_output_name2,
            'archiveWindowLength': 'PT5M',
            'manifestName1': manifest_name1,
            'manifestName2': manifest_name2
        })

        self.cmd('az ams asset create -a {amsname} -n {assetName} -g {rg}')

        self.cmd('az ams live-output create -a {amsname} -n {liveOutputName1} -g {rg} --asset-name {assetName} --live-event-name {liveEventName} --archive-window-length {archiveWindowLength} --manifest-name {manifestName1}')
        self.cmd('az ams live-output create -a {amsname} -n {liveOutputName2} -g {rg} --asset-name {assetName} --live-event-name {liveEventName} --archive-window-length {archiveWindowLength} --manifest-name {manifestName2}')

        self.cmd('az ams live-output list -a {amsname} -g {rg} --live-event-name {liveEventName}', checks=[
            self.check('length(@)', 2)
        ])

        live_event = self.cmd('az ams live-event stop -a {amsname} -n {liveEventName} -g {rg} --remove-outputs-on-stop', checks=[
            self.check('name', '{liveEventName}')
        ]).get_output_in_json()

        self.assertNotEquals('Starting', live_event['resourceState'])
        self.assertNotEquals('Running', live_event['resourceState'])

        self.cmd('az ams live-output list -a {amsname} -g {rg} --live-event-name {liveEventName}', checks=[
            self.check('length(@)', 0)
        ])

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_live_event_list(self, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)
        live_event_name = self.create_random_name(prefix='le', length=12)
        live_event_name2 = self.create_random_name(prefix='le', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'westeurope',
            'streamingProtocol': 'FragmentedMP4',
            'liveEventName': live_event_name,
            'liveEventName2': live_event_name2,
            'encodingType': 'Basic'
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}', checks=[
            self.check('name', '{amsname}'),
            self.check('location', 'West Europe')
        ])

        self.cmd('az ams live-event create -a {amsname} -n {liveEventName} -g {rg} --streaming-protocol {streamingProtocol} --encoding-type {encodingType} --ips AllowAll --tags key=value')

        self.cmd('az ams live-event list -a {amsname} -g {rg}', checks=[
            self.check('length(@)', 1)
        ])

        self.cmd('az ams live-event create -a {amsname} -n {liveEventName2} -g {rg} --streaming-protocol {streamingProtocol} --encoding-type {encodingType} --ips AllowAll --tags key=value')

        self.cmd('az ams live-event list -a {amsname} -g {rg}', checks=[
            self.check('length(@)', 2)
        ])

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_live_event_delete(self, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)
        live_event_name = self.create_random_name(prefix='le', length=12)
        live_event_name2 = self.create_random_name(prefix='le', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'northeurope',
            'streamingProtocol': 'FragmentedMP4',
            'liveEventName': live_event_name,
            'liveEventName2': live_event_name2,
            'encodingType': 'Basic'
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}', checks=[
            self.check('name', '{amsname}'),
            self.check('location', 'North Europe')
        ])

        self.cmd('az ams live-event create -a {amsname} -n {liveEventName} -g {rg} --streaming-protocol {streamingProtocol} --encoding-type {encodingType} --ips AllowAll --tags key=value')

        self.cmd('az ams live-event list -a {amsname} -g {rg}', checks=[
            self.check('length(@)', 1)
        ])

        self.cmd('az ams live-event create -a {amsname} -n {liveEventName2} -g {rg} --streaming-protocol {streamingProtocol} --encoding-type {encodingType} --ips AllowAll --tags key=value')

        self.cmd('az ams live-event list -a {amsname} -g {rg}', checks=[
            self.check('length(@)', 2)
        ])

        self.cmd('az ams live-event delete -a {amsname} -g {rg} -n {liveEventName2}')

        self.cmd('az ams live-event list -a {amsname} -g {rg}', checks=[
            self.check('length(@)', 1)
        ])

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_live_event_reset(self, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)
        live_event_name = self.create_random_name(prefix='le', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'eastus',
            'streamingProtocol': 'FragmentedMP4',
            'liveEventName': live_event_name,
            'encodingType': 'Basic'
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}', checks=[
            self.check('name', '{amsname}'),
            self.check('location', 'East US')
        ])

        self.cmd('az ams live-event create -a {amsname} -n {liveEventName} -g {rg} --streaming-protocol {streamingProtocol} --encoding-type {encodingType} --ips AllowAll --auto-start', checks=[
            self.check('name', '{liveEventName}'),
            self.check('location', 'East US'),
            self.check('input.streamingProtocol', 'FragmentedMP4'),
            self.check('encoding.encodingType', 'Basic')
        ])

        live_event = self.cmd('az ams live-event reset -a {amsname} -n {liveEventName} -g {rg}', checks=[
            self.check('name', '{liveEventName}'),
            self.check('location', 'East US')
        ]).get_output_in_json()

        self.assertNotEquals('Stopping', live_event['resourceState'])
        self.assertNotEquals('Stopped', live_event['resourceState'])

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_live_event_update(self, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)
        live_event_name = self.create_random_name(prefix='le', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'eastasia',
            'streamingProtocol': 'FragmentedMP4',
            'liveEventName': live_event_name
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}')
        self.cmd('az ams live-event create -a {amsname} -n {liveEventName} -g {rg} --ips AllowAll --streaming-protocol {streamingProtocol}')

        self.kwargs.update({
            'tags': 'key=value',
            'keyFrameIntervalDuration': 'PT2S',
            'description': 'asd',
            'clientAccessPolicy': '@' + _get_test_data_file('clientAccessPolicy.xml'),
            'crossDomainPolicy': '@' + _get_test_data_file('crossDomainPolicy.xml')
        })

        live_event_updated = self.cmd('az ams live-event update -a {amsname} -n {liveEventName} -g {rg} --ips 1.2.3.4 5.6.7.8 9.10.11.12 --preview-ips 1.1.1.1 0.0.0.0 --key-frame-interval-duration {keyFrameIntervalDuration} --description {description} --client-access-policy "{clientAccessPolicy}" --cross-domain-policy "{crossDomainPolicy}" --tags {tags}', checks=[
            self.check('description', '{description}'),
            self.check('input.keyFrameIntervalDuration', '{keyFrameIntervalDuration}'),
            self.check('length(preview.accessControl.ip.allow)', 2),
            self.check('length(input.accessControl.ip.allow)', 3),
            self.check('tags.key', 'value')
        ]).get_output_in_json()

        self.assertIsNotNone(live_event_updated['crossSiteAccessPolicies']['crossDomainPolicy'])
        self.assertIsNotNone(live_event_updated['crossSiteAccessPolicies']['clientAccessPolicy'])

    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    def test_live_event_show(self, storage_account_for_create):
        amsname = self.create_random_name(prefix='ams', length=12)
        live_event_name = self.create_random_name(prefix='le', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'westus',
            'streamingProtocol': 'RTMP',
            'liveEventName': live_event_name,
            'encodingType': 'Basic',
            'tags': 'key=value',
            'previewLocator': self.create_guid(),
            'keyFrameIntervalDuration': 'PT2S',
            'description': 'asd',
            'accessToken': '0abf356884d74b4aacbd7b1ebd3da0f7',
            'clientAccessPolicy': '@' + _get_test_data_file('clientAccessPolicy.xml'),
            'crossDomainPolicy': '@' + _get_test_data_file('crossDomainPolicy.xml')
        })

        self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location}', checks=[
            self.check('name', '{amsname}'),
            self.check('location', 'West US')
        ])

        self.cmd('az ams live-event create -a {amsname} -n {liveEventName} -g {rg} --streaming-protocol {streamingProtocol} --encoding-type {encodingType} --tags {tags} --stream-options Default LowLatency --preview-locator {previewLocator} --ips 1.2.3.4 5.6.7.8 9.10.11.12 --preview-ips 1.1.1.1 0.0.0.0 --key-frame-interval-duration {keyFrameIntervalDuration} --access-token {accessToken} --description {description} --client-access-policy "{clientAccessPolicy}" --cross-domain-policy "{crossDomainPolicy}" --vanity-url')

        self.cmd('az ams live-event show -a {amsname} -n {liveEventName} -g {rg}', checks=[
            self.check('name', '{liveEventName}'),
            self.check('location', 'West US'),
            self.check('input.streamingProtocol', '{streamingProtocol}'),
            self.check('encoding.encodingType', '{encodingType}'),
            self.check('length(preview.accessControl.ip.allow)', 2),
            self.check('length(input.accessControl.ip.allow)', 3),
            self.check('preview.previewLocator', '{previewLocator}'),
            self.check('input.keyFrameIntervalDuration', '{keyFrameIntervalDuration}'),
            self.check('length(streamOptions)', 2),
            self.check('description', '{description}'),
            self.check('input.accessToken', '{accessToken}'),
            self.check('vanityUrl', True)
        ])

        nonexits_live_event_name = self.create_random_name(prefix='live-event', length=20)
        self.kwargs.update({
            'nonexits_live_event_name': nonexits_live_event_name
        })
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('az ams live-event show -a {amsname} -n {nonexits_live_event_name} -g {rg}')

        self.cmd('az ams live-event delete -a {amsname} -n {liveEventName} -g {rg}')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest

from azure.cli.core.profiles import ResourceType
from azure.cli.core.mock import DummyCli
from azure.cli.core import AzCommandsLoader
from azure.cli.core.commands import AzCliCommand

from azure.cli.command_modules.resource._color import Color, ColoredStringBuilder
from azure.cli.command_modules.resource._formatters import format_json, format_what_if_operation_result


cli_ctx = DummyCli()
loader = AzCommandsLoader(cli_ctx, resource_type=ResourceType.MGMT_RESOURCE_RESOURCES)
cmd = AzCliCommand(loader, "test", None, resource_type=ResourceType.MGMT_RESOURCE_RESOURCES)

WhatIfOperationResult, WhatIfChange, WhatIfPropertyChange, ChangeType, PropertyChangeType = cmd.get_models(
    "WhatIfOperationResult", "WhatIfChange", "WhatIfPropertyChange", "ChangeType", "PropertyChangeType",
)


class TestFormatJson(unittest.TestCase):
    _colon = ColoredStringBuilder().append(":", Color.RESET).build()
    _left_square_bracket = ColoredStringBuilder().append("[", Color.RESET).build()
    _right_square_bracket = ColoredStringBuilder().append("]", Color.RESET).build()

    def test_leaf(self):
        test_data = (
            ("null", None),
            ("true", True),
            ("false", False),
            ("42", 42),
            ("42.12345", 42.12345),
            ('"foo"', "foo"),
            ("{}", {}),
            ("[]", []),
        )
        for expected, value in test_data:
            with self.subTest(expected=expected, value=value):
                self.assertEqual(expected, format_json(value))

    def test_non_empty_array(self):
        value = [x for x in range(11)]
        expected = """[
  0:  0
  1:  1
  2:  2
  3:  3
  4:  4
  5:  5
  6:  6
  7:  7
  8:  8
  9:  9
  10: 10
]"""
        expected = expected.replace("[", self._left_square_bracket)
        expected = expected.replace("]", self._right_square_bracket)
        expected = expected.replace(":", self._colon)

        self.assertEqual(expected, format_json(value))

    def test_non_empty_object(self):
        value = {"path": {"to": {"foo": "foo"}}, "longPath": {"to": {"bar": "bar"}}}
        expected = """

  path.to.foo:     "foo"
  longPath.to.bar: "bar"
"""
        expected = expected.replace(":", self._colon)

        self.assertEqual(expected, format_json(value))

    def test_complex_value(self):
        value = {
            "root": {
                "foo": 1234,
                "bar": [True, None, {"nestedString": "value", "nestedArray": [92747, "test"]}, [False]],
                "foobar": "foobar",
            }
        }

        expected = """

  root.foo:    1234
  root.bar: [
    0: true
    1: null
    2:

      nestedString: "value"
      nestedArray: [
        0: 92747
        1: "test"
      ]

    3: [
      0: false
    ]
  ]
  root.foobar: "foobar"
"""
        expected = expected.replace("[", self._left_square_bracket)
        expected = expected.replace("]", self._right_square_bracket)
        expected = expected.replace(":", self._colon)

        self.assertEqual(expected, format_json(value))


class TestFormatWhatIfOperationResult(unittest.TestCase):
    def test_change_type_legend(self):
        changes = [
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1/providers/p1/foo1",
                change_type=ChangeType.modify,
            ),
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1/providers/p2/bar",
                change_type=ChangeType.create,
            ),
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000002/resourceGroups/rg2/providers/p1/foo2",
                change_type=ChangeType.modify,
            ),
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000002/providers/p3/foobar1",
                change_type=ChangeType.ignore,
            ),
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000002/resourceGroups/rg3",
                change_type=ChangeType.modify,
                delta=[
                    WhatIfPropertyChange(
                        path="path.to.array.change",
                        property_change_type=PropertyChangeType.array,
                        children=[
                            WhatIfPropertyChange(
                                path="1", property_change_type=PropertyChangeType.delete, before=12345
                            ),
                        ],
                    ),
                ],
            ),
        ]

        expected = f"""Resource and property changes are indicated with these symbols:
  {Color.ORANGE}-{Color.RESET} Delete
  {Color.GREEN}+{Color.RESET} Create
  {Color.PURPLE}~{Color.RESET} Modify
  {Color.GRAY}*{Color.RESET} Ignore
"""

        self.assertIn(expected, format_what_if_operation_result(WhatIfOperationResult(changes=changes)))

    def test_resource_changes_stats(self):
        changes = [
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1/providers/p1/foo1",
                change_type=ChangeType.create,
            ),
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1/providers/p2/bar",
                change_type=ChangeType.create,
            ),
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000002/resourceGroups/rg2/providers/p1/foo2",
                change_type=ChangeType.modify,
            ),
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000002/providers/p3/foobar1",
                change_type=ChangeType.ignore,
            ),
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000002/resourceGroups/rg3",
                change_type=ChangeType.delete,
            ),
        ]

        expected = "\nResource changes: 1 to delete, 2 to create, 1 to modify, 1 to ignore."
        result = format_what_if_operation_result(WhatIfOperationResult(changes=changes))

        self.assertTrue(result.endswith(expected))

    def test_group_resources_changes_by_sorted_scope(self):
        changes = [
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/RG1/providers/p1/foo1",
                change_type=ChangeType.create,
            ),
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1/providers/p2/bar",
                change_type=ChangeType.create,
            ),
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000002/resourceGroups/rg2/providers/p1/foo2",
                change_type=ChangeType.modify,
            ),
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000002/providers/p3/foobar1",
                change_type=ChangeType.ignore,
            ),
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000002/providers/p3/foobar2",
                change_type=ChangeType.delete,
            ),
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000002/resourceGroups/rg3",
                change_type=ChangeType.delete,
            ),
        ]

        expected = f"""
Scope: /subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/RG1
{Color.GREEN}
  + p1/foo1
  + p2/bar
{Color.RESET}
Scope: /subscriptions/00000000-0000-0000-0000-000000000002
{Color.ORANGE}
  - p3/foobar2
  - resourceGroups/rg3{Color.RESET}{Color.GRAY}
  * p3/foobar1
{Color.RESET}
Scope: /subscriptions/00000000-0000-0000-0000-000000000002/resourceGroups/rg2
{Color.PURPLE}
  ~ p1/foo2
{Color.RESET}"""

        result = format_what_if_operation_result(WhatIfOperationResult(changes=changes))

        self.assertIn(expected, result)

    def test_sort_resource_ids_within_a_scope(self):
        changes = [
            WhatIfChange(
                resource_id="subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1/providers/p1/foo",
                change_type=ChangeType.ignore,
            ),
            WhatIfChange(
                resource_id="subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1/providers/p2/foo",
                change_type=ChangeType.create,
            ),
            WhatIfChange(
                resource_id="subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1/providers/p3/foo",
                change_type=ChangeType.no_change,
            ),
            WhatIfChange(
                resource_id="subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1/providers/p4/foo",
                change_type=ChangeType.deploy,
            ),
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1/providers/p5/foo",
                change_type=ChangeType.delete,
            ),
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1/providers/p6/foo",
                change_type=ChangeType.delete,
            ),
            WhatIfChange(
                resource_id="/subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1/providers/p7/foo",
                change_type=ChangeType.delete,
            ),
        ]

        expected = f"""
Scope: /subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1
{Color.ORANGE}
  - p5/foo
  - p6/foo
  - p7/foo{Color.RESET}{Color.GREEN}
  + p2/foo{Color.RESET}{Color.BLUE}
  ! p4/foo{Color.RESET}{Color.RESET}
  = p3/foo{Color.RESET}{Color.GRAY}
  * p1/foo
{Color.RESET}"""
        result = format_what_if_operation_result(WhatIfOperationResult(changes=changes))

        self.assertIn(expected, result)

    def test_property_create(self):
        changes = [
            WhatIfChange(
                resource_id="subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1/providers/p1/foo",
                change_type=ChangeType.create,
                after={
                    "numberValue": 1.2,
                    "booleanValue": True,
                    "stringValue": "The quick brown fox jumps over the lazy dog.",
                },
            ),
        ]

        expected = f"""
Scope: /subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1
{Color.GREEN}
  + p1/foo

      numberValue{Color.RESET}:{Color.GREEN}  1.2
      booleanValue{Color.RESET}:{Color.GREEN} true
      stringValue{Color.RESET}:{Color.GREEN}  "The quick brown fox jumps over the lazy dog."
{Color.RESET}"""

        result = format_what_if_operation_result(WhatIfOperationResult(changes=changes))

        self.assertIn(expected, result)

    def test_property_delete(self):
        changes = [
            WhatIfChange(
                resource_id="subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1/providers/p1/foo",
                change_type=ChangeType.delete,
                before={
                    "apiVersion": "2020-04-01",
                    "numberValue": 1.2,
                    "booleanValue": True,
                    "stringValue": "The quick brown fox jumps over the lazy dog.",
                },
            ),
        ]

        expected = f"""
Scope: /subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1
{Color.ORANGE}
  - p1/foo{Color.RESET} [2020-04-01]{Color.ORANGE}

      apiVersion{Color.RESET}:{Color.ORANGE}   "2020-04-01"
      numberValue{Color.RESET}:{Color.ORANGE}  1.2
      booleanValue{Color.RESET}:{Color.ORANGE} true
      stringValue{Color.RESET}:{Color.ORANGE}  "The quick brown fox jumps over the lazy dog."
{Color.RESET}"""

        result = format_what_if_operation_result(WhatIfOperationResult(changes=changes))

        self.assertIn(expected, result)

    def test_property_modify(self):
        changes = [
            WhatIfChange(
                resource_id="subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1/providers/p1/foo",
                change_type=ChangeType.modify,
                delta=[
                    WhatIfPropertyChange(
                        path="path.a.to.change",
                        property_change_type=PropertyChangeType.modify,
                        before="foo",
                        after="bar",
                    ),
                    WhatIfPropertyChange(
                        path="path.b.to.nested.change",
                        property_change_type=PropertyChangeType.array,
                        children=[
                            WhatIfPropertyChange(
                                path="4",
                                property_change_type=PropertyChangeType.modify,
                                children=[
                                    WhatIfPropertyChange(
                                        path="foo.bar",
                                        property_change_type=PropertyChangeType.modify,
                                        before=True,
                                        after=False,
                                    ),
                                    WhatIfPropertyChange(
                                        path="baz",
                                        property_change_type=PropertyChangeType.create,
                                        after=["element1", "element2"],
                                    ),
                                ],
                            ),
                            WhatIfPropertyChange(
                                path="5", property_change_type=PropertyChangeType.delete, before=12345
                            ),
                        ],
                    ),
                ],
            ),
        ]

        expected = f"""
Scope: /subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1
{Color.PURPLE}
  ~ p1/foo{Color.RESET}
    {Color.PURPLE}~{Color.RESET} path.a.to.change{Color.RESET}:{Color.RESET} {Color.ORANGE}"foo"{Color.RESET} => {Color.GREEN}"bar"{Color.RESET}
    {Color.PURPLE}~{Color.RESET} path.b.to.nested.change{Color.RESET}:{Color.RESET} [
      {Color.PURPLE}~{Color.RESET} 4{Color.RESET}:{Color.RESET}

        {Color.GREEN}+{Color.RESET} baz{Color.RESET}:{Color.RESET} {Color.GREEN}{Color.RESET}[{Color.GREEN}
            0{Color.RESET}:{Color.GREEN} "element1"
            1{Color.RESET}:{Color.GREEN} "element2"
          {Color.RESET}]{Color.GREEN}{Color.RESET}
        {Color.PURPLE}~{Color.RESET} foo.bar{Color.RESET}:{Color.RESET} {Color.ORANGE}true{Color.RESET} => {Color.GREEN}false{Color.RESET}

      {Color.ORANGE}-{Color.RESET} 5{Color.RESET}:{Color.RESET} {Color.ORANGE}12345{Color.RESET}
      ]
{Color.PURPLE}{Color.RESET}"""

        result = format_what_if_operation_result(WhatIfOperationResult(changes=changes))

        self.assertIn(expected, result)

    def test_json_alignment(self):
        changes = [
            WhatIfChange(
                resource_id="subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1/providers/p1/foo",
                change_type=ChangeType.delete,
                before={
                    "apiVersion": "2020-04-01",
                    "numberValue": 1.2,
                    "booleanValue": True,
                    "stringValue": "The quick brown fox jumps over the lazy dog.",
                    "emptyArray": [],
                    "emptyObject": {},
                    "arrayContaingValues": ["foo", "bar"],
                },
            ),
        ]

        expected = """
Scope: /subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1

  - p1/foo [2020-04-01]

      apiVersion:   "2020-04-01"
      numberValue:  1.2
      booleanValue: true
      stringValue:  "The quick brown fox jumps over the lazy dog."
      emptyArray:   []
      emptyObject:  {}
      arrayContaingValues: [
        0: "foo"
        1: "bar"
      ]
"""

        result = format_what_if_operation_result(WhatIfOperationResult(changes=changes), False)

        self.assertIn(expected, result)

    def test_property_changes_alignment(self):
        changes = [
            WhatIfChange(
                resource_id="subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1/providers/p1/foo",
                change_type=ChangeType.modify,
                delta=[
                    WhatIfPropertyChange(path="path", property_change_type=PropertyChangeType.delete, before={},),
                    WhatIfPropertyChange(path="long.path", property_change_type=PropertyChangeType.create, after=[],),
                    WhatIfPropertyChange(
                        path="long.nested.path",
                        property_change_type=PropertyChangeType.array,
                        children=[
                            WhatIfPropertyChange(
                                path="5", property_change_type=PropertyChangeType.delete, before=12345
                            ),
                        ],
                    ),
                ],
            ),
        ]

        expected = """
Scope: /subscriptions/00000000-0000-0000-0000-000000000001/resourceGroups/rg1

  ~ p1/foo
    - path:      {}
    + long.path: []
    ~ long.nested.path: [
      - 5: 12345
      ]
"""

        result = format_what_if_operation_result(WhatIfOperationResult(changes=changes), False)

        self.assertIn(expected, result)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import datetime
import time

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer
from knack.util import CLIError
from .batch_preparers import BatchAccountPreparer, BatchScenarioMixin


class BatchDataPlaneScenarioTests(BatchScenarioMixin, ScenarioTest):

    def _get_test_data_file(self, filename):
        filepath = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data', filename)
        self.assertTrue(os.path.isfile(filepath), 'File {} does not exist.'.format(filepath))
        return filepath

    @ResourceGroupPreparer()
    @BatchAccountPreparer(location='northcentralus')
    def test_batch_certificate_cmd(self, resource_group, batch_account_name):
        create_cert_file_path = self._get_test_data_file('batchtest.cer')
        self.kwargs.update({
            'cert': '59833fd835f827e9ec693a4c82435a6360cc6271',
            'cert_f': create_cert_file_path
        })

        # test create certificate with default set
        self.set_account_info(batch_account_name, resource_group)

        self.batch_cmd('batch certificate create --thumbprint {cert} '
                       '--certificate-file "{cert_f}"').assert_with_checks([
                           self.check('thumbprint', '{cert}'),
                           self.check('thumbprintAlgorithm', 'sha1'),
                           self.check('state', 'active')])

        # test create account with default set
        self.batch_cmd('batch certificate list').assert_with_checks([
            self.check('length(@)', 1),
            self.check('[0].thumbprint', '{cert}')])

        self.batch_cmd("batch certificate delete --thumbprint {cert} --yes")

        self.batch_cmd('batch certificate show --thumbprint {cert}').assert_with_checks([
            self.check('thumbprint', '{cert}'),
            self.check('thumbprintAlgorithm', 'sha1'),
            self.check('state', 'deleting')])

    @ResourceGroupPreparer()
    @BatchAccountPreparer()
    def test_batch_pool_cmd(
            self,
            resource_group,
            batch_account_name):
        endpoint = self.get_account_endpoint(
            batch_account_name,
            resource_group).replace("https://", "")
        key = self.get_account_key(
            batch_account_name,
            resource_group)

        self.kwargs.update({
            'p_id': 'xplatCreatedPool',
            'c_file': self._get_test_data_file('batchCreatePool.json'),
            'u_file': self._get_test_data_file('batchUpdatePool.json'),
            'acc_n': batch_account_name,
            'acc_k': key,
            'acc_u': endpoint
        })
        self.batch_cmd('batch pool create --json-file "{c_file}"')
        result = self.batch_cmd('batch pool create --id pool_image1 --vm-size Standard_A1 '
                                '--image a:b:c --node-agent-sku-id "batch.node.windows amd64"',
                                expect_failure=True)

        result = self.batch_cmd('batch pool create --id pool_image1 --vm-size Standard_A1 '
                                '--image /subscriptions/11111111-1111-1111-1111-111111111111'
                                '/resourceGroups/test_rg/providers/Microsoft.Compute/images/custom_image '
                                '--node-agent-sku-id "batch.node.windows amd64"',
                                expect_failure=True)

        result = self.batch_cmd('batch pool create --id pool_image1 --vm-size Standard_A1 '
                                '--image canonical:ubuntuserver:18.04-lts --node-agent-sku-id "batch.node.ubuntu 18.04"'
                                ' --disk-encryption-targets "TemporaryDisk"')

        time.sleep(120)

        result = self.batch_cmd('batch pool show --pool-id {p_id}').assert_with_checks([
            self.check('allocationState', 'steady'),
            self.check('id', 'xplatCreatedPool'),
            self.check('startTask.commandLine', "cmd /c echo test"),
            self.check('startTask.userIdentity.autoUser.elevationLevel', "admin")])

        target = result.get_output_in_json()['currentLowPriorityNodes']
        self.batch_cmd('batch pool resize --pool-id {p_id} --target-dedicated-nodes 0 --target-low-priority-nodes 3')
        self.batch_cmd('batch pool show --pool-id {p_id}').assert_with_checks([
            self.check('allocationState', 'resizing'),
            self.check('targetLowPriorityNodes', 3),
            self.check('id', 'xplatCreatedPool')])

        self.batch_cmd('batch pool node-counts list').assert_with_checks([
            self.check('length(@)', 2),
            self.check('[1].poolId', 'xplatCreatedPool'),
            self.check('[1].lowPriority.total', 0)])

        self.batch_cmd('batch pool resize --pool-id {p_id} --abort')
        if self.is_live or self.in_recording:
            time.sleep(120)

        self.batch_cmd('batch pool show --pool-id {p_id}').assert_with_checks([
            self.check('allocationState', 'steady'),
            self.check('id', 'xplatCreatedPool'),
            self.check('currentLowPriorityNodes', target),
            self.check('targetLowPriorityNodes', 3)])

        self.batch_cmd('batch pool reset --pool-id {p_id} --json-file "{u_file}"').assert_with_checks([
            self.check('allocationState', 'steady'),
            self.check('id', 'xplatCreatedPool'),
            self.check('startTask.commandLine', "cmd /c echo updated")])

        self.batch_cmd('batch pool reset --pool-id {p_id} --start-task-command-line hostname '
                       '--metadata a=b c=d').assert_with_checks([
                           self.check('allocationState', 'steady'),
                           self.check('id', 'xplatCreatedPool'),
                           self.check('startTask.commandLine', "hostname"),
                           self.check('length(metadata)', 2),
                           self.check('metadata[0].name', 'a'),
                           self.check('metadata[1].value', 'd')])

        self.batch_cmd('batch pool delete --pool-id {p_id} --yes')

    @ResourceGroupPreparer()
    @BatchAccountPreparer()
    def test_batch_job_list_cmd(
            self,
            resource_group,
            batch_account_name):
        self.set_account_info(batch_account_name, resource_group)
        self.kwargs.update({
            'j_id': 'xplatJob',
            'js_id': 'xplatJobScheduleJobTests',
            'j_file': self._get_test_data_file('batchCreateJob.json'),
            'js_file': self._get_test_data_file('batchCreateJobScheduleForJobTests.json')
        })

        self.batch_cmd('batch job-schedule create --json-file "{js_file}"')
        self.addCleanup(lambda: self.batch_cmd('batch job-schedule delete --job-schedule-id {js_id} --yes'))

        self.batch_cmd('batch job create --json-file "{j_file}"')
        self.batch_cmd('batch job list --job-schedule-id {js_id}').assert_with_checks([
            self.check('length(@)', 1),
            self.check('[0].id', 'xplatJobScheduleJobTests:job-1')])

        result = self.batch_cmd('batch job list') \
            .assert_with_checks([self.check('length(@)', 2)]) \
            .get_output_in_json()

        self.assertTrue(any([i for i in result if i['id'] == 'xplatJobScheduleJobTests:job-1']))
        self.assertTrue(any([i for i in result if i['id'] == 'xplatJob']))

        self.batch_cmd('batch job delete --job-id {j_id} --yes')

    @ResourceGroupPreparer()
    @BatchAccountPreparer(location='canadacentral')
    def test_batch_task_create_cmd(self, resource_group, batch_account_name):
        self.set_account_info(batch_account_name, resource_group)
        self.kwargs.update({
            'j_id': 'xplatJobForTaskTests',
            't_id': 'xplatTask',
            'j_file': self._get_test_data_file('batchCreateJobForTaskTests.json'),
            't_file': self._get_test_data_file('batchCreateTask.json'),
            'ts_file': self._get_test_data_file('batchCreateMultiTasks.json')
        })

        self.batch_cmd('batch job create --json-file "{j_file}"')
        self.addCleanup(lambda: self.batch_cmd('batch job delete --job-id {j_id} --yes'))

        self.batch_cmd('batch task create --job-id {j_id} --json-file "{t_file}"').assert_with_checks([
            self.check('id', 'xplatTask'),
            self.check('commandLine', 'cmd /c dir /s')])

        self.batch_cmd('batch task show --job-id {j_id} --task-id {t_id}').assert_with_checks([
            self.check('userIdentity.autoUser.scope', 'pool'),
            self.check('authenticationTokenSettings.access[0]', 'job')])

        self.batch_cmd('batch task delete --job-id {j_id} --task-id {t_id} --yes')

        self.batch_cmd('batch task create --job-id {j_id} --task-id aaa'
                       ' --command-line "ping 127.0.0.1 -n 30"').assert_with_checks([
                           self.check('id', 'aaa'),
                           self.check('commandLine', 'ping 127.0.0.1 -n 30')])

        if self.is_live or self.in_recording:
            time.sleep(10)
        task_result = self.batch_cmd('batch job task-counts show --job-id {j_id}').get_output_in_json()

        self.assertEqual(task_result["taskCounts"]["completed"], 0)
        self.assertEqual(task_result["taskCounts"]["active"], 1)
        self.assertEqual(task_result["taskSlotCounts"]["completed"], 0)
        self.assertEqual(task_result["taskSlotCounts"]["active"], 1)

        self.batch_cmd('batch task delete --job-id {j_id} --task-id aaa --yes')

        result = self.batch_cmd('batch task create --job-id {j_id} --json-file "{ts_file}"')
        result = result.get_output_in_json()
        self.assertEqual(len(result), 3)
        self.assertTrue(any([i for i in result if i['taskId'] == 'xplatTask1']))

    @ResourceGroupPreparer()
    @BatchAccountPreparer(location='canadaeast')
    def test_batch_jobs_and_tasks(self, resource_group, batch_account_name):
        self.set_account_info(batch_account_name, resource_group)
        self.kwargs.update({
            'p_id': 'xplatJobForTaskTests',
            'j_id': "cli-test-job-1",
        })

        # test create paas pool using parameters
        self.batch_cmd('batch pool create --id {p_id} --vm-size small --os-family 4')

        # test create job with missing parameters
        self.kwargs['start'] = datetime.datetime.now().isoformat()
        with self.assertRaises(SystemExit):
            self.batch_cmd('batch job create --id {j_id} --metadata test=value '
                           '--job-manager-task-environment-settings a=b '
                           '--job-max-task-retry-count 5 ')

        # test create job
        self.batch_cmd('batch job create --id {j_id} --metadata test=value '
                       '--job-max-task-retry-count 5 '
                       '--job-manager-task-id JobManager '
                       '--job-manager-task-command-line "cmd /c set AZ_BATCH_TASK_ID" '
                       '--job-manager-task-environment-settings '
                       'CLI_TEST_VAR=CLI_TEST_VAR_VALUE --pool-id {p_id}')

        # test get job
        self.batch_cmd('batch job show --job-id {j_id}').assert_with_checks([
            self.check('onAllTasksComplete', 'noaction'),
            self.check('constraints.maxTaskRetryCount', 5),
            self.check('jobManagerTask.id', 'JobManager'),
            self.check('jobManagerTask.environmentSettings[0].name', 'CLI_TEST_VAR'),
            self.check('jobManagerTask.environmentSettings[0].value', 'CLI_TEST_VAR_VALUE'),
            self.check('metadata[0].name', 'test'),
            self.check('metadata[0].value', 'value')])

        # test bad enum value
        with self.assertRaises(SystemExit):
            self.batch_cmd('batch job set --job-id {j_id} --on-all-tasks-complete badValue ')

        # test patch job
        self.batch_cmd('batch job set --job-id {j_id} --job-max-wall-clock-time P3Y6M4DT12H30M5S '
                       '--on-all-tasks-complete terminatejob')
        self.batch_cmd('batch job show --job-id {j_id}').assert_with_checks([
            self.check('onAllTasksComplete', 'terminatejob'),
            self.check('constraints.maxTaskRetryCount', 0),
            self.check('constraints.maxWallClockTime', '1279 days, 12:30:05'),
            self.check('jobManagerTask.id', 'JobManager'),
            self.check('jobManagerTask.environmentSettings[0].name', 'CLI_TEST_VAR'),
            self.check('jobManagerTask.environmentSettings[0].value', 'CLI_TEST_VAR_VALUE'),
            self.check('metadata[0].name', 'test'),
            self.check('metadata[0].value', 'value')])

        # test filter/header argument
        self.batch_cmd('batch job reset --job-id {j_id} --pool-id {p_id} --on-all-tasks-complete '
                       'terminatejob --if-unmodified-since {start}', expect_failure=True)

        # test reset job
        self.batch_cmd('batch job reset --job-id {j_id} --pool-id {p_id}  '
                       '--on-all-tasks-complete terminatejob ')
        job = self.batch_cmd('batch job show --job-id {j_id}').assert_with_checks([
            self.check('constraints.maxTaskRetryCount', 0),
            self.check('metadata', None)])

        job = job.get_output_in_json()
        self.assertFalse(job['constraints']['maxWallClockTime'] == '1279 days, 12:30:05')

        # TODO: test task commands

    @ResourceGroupPreparer()
    @BatchAccountPreparer()
    def test_batch_pools_and_nodes(
            self,
            resource_group,
            batch_account_name):  # pylint:disable=too-many-statements
        self.set_account_info(batch_account_name, resource_group)
        self.kwargs.update({
            'pool_p': "azure-cli-test-paas",
            'pool_i': "azure-cli-test-iaas",
            'pool_j': "azure-cli-test-json"
        })

        # test create paas pool using parameters
        self.batch_cmd('batch pool create --id {pool_p} --vm-size small --os-family 4')

        # test create iaas pool using parameters
        self.batch_cmd('batch pool create --id {pool_i} --vm-size Standard_A1 '
                       '--image Canonical:UbuntuServer:16.04.0-LTS '
                       '--node-agent-sku-id "batch.node.ubuntu 16.04"')

        # test create pool with missing parameters
        with self.assertRaises(SystemExit):
            self.batch_cmd('batch pool create --id missing-params-test --os-family 4')

        # test create pool with missing required mutually exclusive parameters
        with self.assertRaises(SystemExit):
            self.batch_cmd('batch pool create --id missing-required-group-test --vm-size small')

        # test create pool with parameters from mutually exclusive groups
        with self.assertRaises(SystemExit):
            self.batch_cmd('batch pool create --id mutually-exclusive-test --vm-size small '
                           '--os-family 4 --image Canonical:UbuntuServer:16-LTS:latest')

        # test create pool with invalid vm size for IaaS
        with self.assertRaises(SystemExit):
            self.batch_cmd('batch pool create --id invalid-size-test --vm-size small '
                           '--image Canonical:UbuntuServer:16.04.0-LTS --node-agent-sku-id '
                           '"batch.node.ubuntu 16.04"')

        # test create pool with missing optional parameters
        with self.assertRaises(SystemExit):
            self.batch_cmd('batch pool create --id missing-optional-test --vm-size small '
                           '--os-family 4 --start-task-wait-for-success')

        # test create pool from JSON file
        self.kwargs['json'] = self._get_test_data_file('batch-pool-create.json').replace('\\', '\\\\')
        self.batch_cmd('batch pool create --json-file "{json}"')
        self.batch_cmd('batch pool show --pool-id azure-cli-test-json').assert_with_checks([
            self.check('userAccounts[0].name', 'cliTestUser'),
            self.check('startTask.userIdentity.userName', 'cliTestUser'),
            self.check('taskSlotsPerNode', 3)
        ])

        # test create pool from non-existant JSON file
        with self.assertRaises(SystemExit):
            self.batch_cmd('batch pool create --json-file batch-pool-create-missing.json')

        # test create pool from invalid JSON file
        with self.assertRaises(CLIError):
            self.kwargs['json'] = self._get_test_data_file('batch-pool-create-invalid.json').replace('\\', '\\\\')
            self.batch_cmd('batch pool create --json-file "{json}"')

        # test create pool from JSON file with additional parameters
        with self.assertRaises(SystemExit):
            self.kwargs['json'] = self._get_test_data_file('batch-pool-create.json').replace('\\', '\\\\')
            self.batch_cmd('batch pool create --json-file "{json}" --vm-size small')

        # test list pools
        pool_list = self.batch_cmd('batch pool list')
        pool_list = pool_list.get_output_in_json()
        self.assertEqual(len(pool_list), 3)
        pool_ids = sorted([p['id'] for p in pool_list])
        self.assertEqual(pool_ids, ["azure-cli-test-iaas", "azure-cli-test-json", "azure-cli-test-paas"])

        # test list pools with select
        pool_list = self.batch_cmd('batch pool list --filter "id eq \'{pool_p}\'"').get_output_in_json()
        self.assertEqual(len(pool_list), 1)

        # test resize pool
        self.batch_cmd('batch pool resize --pool-id {pool_p} --target-dedicated-nodes 0 --target-low-priority-nodes 5')
        self.batch_cmd('batch pool show --pool-id {pool_p} '
                       '--select "allocationState, targetLowPriorityNodes"').assert_with_checks([
                           self.check('targetLowPriorityNodes', 5)])

        # test cancel pool resize
        self.batch_cmd('batch pool resize --pool-id {pool_p} --abort')

        # test enable autoscale
        self.batch_cmd('batch pool autoscale enable --pool-id {pool_i} '
                       '--auto-scale-formula "$TargetLowPriorityNodes=3"')
        self.batch_cmd('batch pool show --pool-id {pool_i} --select "enableAutoScale"').assert_with_checks([
            self.check('enableAutoScale', True)])

        # test evaluate autoscale
        self.batch_cmd('batch pool autoscale evaluate --pool-id {pool_i} '
                       '--auto-scale-formula "$TargetLowPriorityNodes=3"')

        # test disable autoscale
        self.batch_cmd('batch pool autoscale disable --pool-id {pool_i}')
        self.batch_cmd('batch pool show --pool-id {pool_i} --select "enableAutoScale"').assert_with_checks([
            self.check('enableAutoScale', False)])

        # test list usage metrics
        self.batch_cmd('batch pool usage-metrics list')

        # TODO: Test update pool from JSON file

        # test patch pool using parameters
        current = self.batch_cmd('batch pool show --pool-id {pool_j} --select "startTask"').get_output_in_json()
        self.batch_cmd('batch pool set --pool-id {pool_j} --start-task-command-line new_value')
        updated = self.batch_cmd('batch pool show --pool-id {pool_j} --select "startTask"').get_output_in_json()
        self.assertNotEqual(current['startTask']['commandLine'], updated['startTask']['commandLine'])

        # test list node agent skus
        self.batch_cmd('batch pool supported-images list')

        # test delete iaas pool
        self.batch_cmd('batch pool delete --pool-id {pool_i} --yes')

        # test app package reference
        self.batch_cmd('batch pool create --id app_package_test --vm-size small --os-family 4 '
                       '--application-package-references does-not-exist', expect_failure=True)
        self.batch_cmd('batch pool set --pool-id {pool_p} --application-package-references does-not-exist',
                       expect_failure=True)

        # TODO: Test node commands



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import zipfile
from knack.util import CLIError
from knack.log import get_logger
from azure.cli.core.commands.client_factory import get_mgmt_service_client
from azure.cli.core.util import get_file_json
from azure.mgmt.web.models import SkuDescription

from ._constants import (NETCORE_VERSION_DEFAULT, NETCORE_VERSIONS, NODE_VERSION_DEFAULT,
                         NODE_VERSIONS, NETCORE_RUNTIME_NAME, NODE_RUNTIME_NAME, ASPDOTNET_RUNTIME_NAME,
                         ASPDOTNET_VERSION_DEFAULT, DOTNET_VERSIONS, STATIC_RUNTIME_NAME,
                         PYTHON_RUNTIME_NAME, PYTHON_VERSION_DEFAULT, LINUX_SKU_DEFAULT, OS_DEFAULT,
                         NODE_VERSION_NEWER, DOTNET_RUNTIME_NAME, DOTNET_VERSION_DEFAULT,
                         DOTNET_TARGET_FRAMEWORK_STRING, GENERATE_RANDOM_APP_NAMES)

logger = get_logger(__name__)


def _resource_client_factory(cli_ctx, **_):
    from azure.cli.core.profiles import ResourceType
    return get_mgmt_service_client(cli_ctx, ResourceType.MGMT_RESOURCE_RESOURCES)


def web_client_factory(cli_ctx, **_):
    from azure.mgmt.web import WebSiteManagementClient
    return get_mgmt_service_client(cli_ctx, WebSiteManagementClient)


def zip_contents_from_dir(dirPath, lang):
    import tempfile
    import uuid
    relroot = os.path.abspath(tempfile.gettempdir())
    path_and_file = os.path.splitdrive(dirPath)[1]
    file_val = os.path.split(path_and_file)[1]
    file_val_unique = file_val + str(uuid.uuid4())[:259]
    zip_file_path = relroot + os.path.sep + file_val_unique + ".zip"
    abs_src = os.path.abspath(dirPath)
    try:
        with zipfile.ZipFile("{}".format(zip_file_path), "w", zipfile.ZIP_DEFLATED) as zf:
            for dirname, subdirs, files in os.walk(dirPath):
                # skip node_modules folder for Node apps,
                # since zip_deployment will perform the build operation
                if lang.lower() == NODE_RUNTIME_NAME:
                    subdirs[:] = [d for d in subdirs if 'node_modules' not in d]
                elif lang.lower() == NETCORE_RUNTIME_NAME:
                    subdirs[:] = [d for d in subdirs if d not in ['obj', 'bin']]
                elif lang.lower() == PYTHON_RUNTIME_NAME:
                    subdirs[:] = [d for d in subdirs if 'env' not in d]  # Ignores dir that contain env

                    filtered_files = []
                    for filename in files:
                        if filename == '.env':
                            logger.info("Skipping file: %s/%s", dirname, filename)
                        else:
                            filtered_files.append(filename)
                    files[:] = filtered_files

                for filename in files:
                    absname = os.path.abspath(os.path.join(dirname, filename))
                    arcname = absname[len(abs_src) + 1:]
                    zf.write(absname, arcname)
    except IOError as e:
        if e.errno == 13:
            raise CLIError('Insufficient permissions to create a zip in current directory. '
                           'Please re-run the command with administrator privileges')
        raise CLIError(e)

    return zip_file_path


def get_runtime_version_details(file_path, lang_name):
    version_detected = None
    version_to_create = None
    if lang_name.lower() == DOTNET_RUNTIME_NAME:
        version_detected = DOTNET_VERSION_DEFAULT
        version_to_create = DOTNET_VERSION_DEFAULT
    elif lang_name.lower() == NETCORE_RUNTIME_NAME:
        # method returns list in DESC, pick the first
        version_detected = parse_netcore_version(file_path)[0]
        version_to_create = detect_netcore_version_tocreate(version_detected)
    elif lang_name.lower() == ASPDOTNET_RUNTIME_NAME:
        # method returns list in DESC, pick the first
        version_detected = parse_dotnet_version(file_path)
        version_to_create = detect_dotnet_version_tocreate(version_detected)
    elif lang_name.lower() == NODE_RUNTIME_NAME:
        if file_path == '':
            version_detected = "-"
            version_to_create = NODE_VERSION_DEFAULT
        else:
            version_detected = parse_node_version(file_path)[0]
            version_to_create = detect_node_version_tocreate(version_detected)
    elif lang_name.lower() == PYTHON_RUNTIME_NAME:
        version_detected = "-"
        version_to_create = PYTHON_VERSION_DEFAULT
    elif lang_name.lower() == STATIC_RUNTIME_NAME:
        version_detected = "-"
        version_to_create = "-"
    return {'detected': version_detected, 'to_create': version_to_create}


def create_resource_group(cmd, rg_name, location):
    from azure.cli.core.profiles import ResourceType, get_sdk
    rcf = _resource_client_factory(cmd.cli_ctx)
    resource_group = get_sdk(cmd.cli_ctx, ResourceType.MGMT_RESOURCE_RESOURCES, 'ResourceGroup', mod='models')
    rg_params = resource_group(location=location)
    return rcf.resource_groups.create_or_update(rg_name, rg_params)


def _check_resource_group_exists(cmd, rg_name):
    rcf = _resource_client_factory(cmd.cli_ctx)
    return rcf.resource_groups.check_existence(rg_name)


def _check_resource_group_supports_os(cmd, rg_name, is_linux):
    # get all appservice plans from RG
    client = web_client_factory(cmd.cli_ctx)
    plans = list(client.app_service_plans.list_by_resource_group(rg_name))
    for item in plans:
        # for Linux if an app with reserved==False exists, ASP doesn't support Linux
        if is_linux and not item.reserved:
            return False
        if not is_linux and item.reserved:
            return False
    return True


def get_num_apps_in_asp(cmd, rg_name, asp_name):
    client = web_client_factory(cmd.cli_ctx)
    return len(list(client.app_service_plans.list_web_apps(rg_name, asp_name)))


# pylint:disable=unexpected-keyword-arg
def get_lang_from_content(src_path, html=False):
    # NODE: package.json should exist in the application root dir
    # NETCORE & DOTNET: *.csproj should exist in the application dir
    # NETCORE: <TargetFramework>netcoreapp2.0</TargetFramework>
    # DOTNET: <TargetFrameworkVersion>v4.5.2</TargetFrameworkVersion>
    runtime_details_dict = dict.fromkeys(['language', 'file_loc', 'default_sku'])
    package_json_file = os.path.join(src_path, 'package.json')
    package_python_file = os.path.join(src_path, 'requirements.txt')
    static_html_file = ""
    package_netcore_file = ""
    runtime_details_dict['language'] = ''
    runtime_details_dict['file_loc'] = ''
    runtime_details_dict['default_sku'] = 'F1'
    import fnmatch
    for _dirpath, _dirnames, files in os.walk(src_path):
        for file in files:
            if html and (fnmatch.fnmatch(file, "*.html") or fnmatch.fnmatch(file, "*.htm") or
                         fnmatch.fnmatch(file, "*shtml.")):
                static_html_file = os.path.join(src_path, file)
                break
            elif fnmatch.fnmatch(file, "*.csproj"):
                package_netcore_file = os.path.join(src_path, file)
                break

    if html:
        if static_html_file:
            runtime_details_dict['language'] = STATIC_RUNTIME_NAME
            runtime_details_dict['file_loc'] = static_html_file
            runtime_details_dict['default_sku'] = 'F1'
        else:
            raise CLIError("The html flag was passed, but could not find HTML files, "
                           "see 'https://go.microsoft.com/fwlink/?linkid=2109470' for more information")
    elif os.path.isfile(package_python_file):
        runtime_details_dict['language'] = PYTHON_RUNTIME_NAME
        runtime_details_dict['file_loc'] = package_python_file
        runtime_details_dict['default_sku'] = LINUX_SKU_DEFAULT
    elif os.path.isfile(package_json_file) or os.path.isfile('server.js') or os.path.isfile('index.js'):
        runtime_details_dict['language'] = NODE_RUNTIME_NAME
        runtime_details_dict['file_loc'] = package_json_file if os.path.isfile(package_json_file) else ''
        runtime_details_dict['default_sku'] = LINUX_SKU_DEFAULT
    elif package_netcore_file:
        runtime_lang = detect_dotnet_lang(package_netcore_file)
        runtime_details_dict['language'] = runtime_lang
        runtime_details_dict['file_loc'] = package_netcore_file
        runtime_details_dict['default_sku'] = 'F1'
    else:  # TODO: Update the doc when the detection logic gets updated
        raise CLIError("Could not auto-detect the runtime stack of your app.\n"
                       "HINT: Are you in the right folder?\n"
                       "For more information, see 'https://go.microsoft.com/fwlink/?linkid=2109470'")
    return runtime_details_dict


def detect_dotnet_lang(csproj_path):
    import xml.etree.ElementTree as ET
    import re
    parsed_file = ET.parse(csproj_path)
    root = parsed_file.getroot()
    version_lang = ''
    version_full = ''
    for target_ver in root.iter('TargetFramework'):
        version_full = target_ver.text
        version_full = ''.join(version_full.split()).lower()
        version_lang = re.sub(r'([^a-zA-Z\s]+?)', '', target_ver.text)

    if version_full and version_full.startswith(DOTNET_TARGET_FRAMEWORK_STRING):
        return DOTNET_RUNTIME_NAME
    if 'netcore' in version_lang.lower():
        return NETCORE_RUNTIME_NAME
    return ASPDOTNET_RUNTIME_NAME


def parse_dotnet_version(file_path):
    version_detected = ['4.7']
    try:
        from xml.dom import minidom
        import re
        xmldoc = minidom.parse(file_path)
        framework_ver = xmldoc.getElementsByTagName('TargetFrameworkVersion')
        target_ver = framework_ver[0].firstChild.data
        non_decimal = re.compile(r'[^\d.]+')
        # reduce the version to '5.7.4' from '5.7'
        if target_ver is not None:
            # remove the string from the beginning of the version value
            c = non_decimal.sub('', target_ver)
            version_detected = c[:3]
    except:  # pylint: disable=bare-except
        version_detected = version_detected[0]
    return version_detected


def parse_netcore_version(file_path):
    import xml.etree.ElementTree as ET
    import re
    version_detected = ['0.0']
    parsed_file = ET.parse(file_path)
    root = parsed_file.getroot()
    for target_ver in root.iter('TargetFramework'):
        version_detected = re.findall(r"\d+\.\d+", target_ver.text)
    # incase of multiple versions detected, return list in descending order
    version_detected = sorted(version_detected, key=float, reverse=True)
    return version_detected


def parse_node_version(file_path):
    # from node experts the node value in package.json can be found here   "engines": { "node":  ">=10.6.0"}
    import json
    import re
    version_detected = []
    with open(file_path) as data_file:
        data = json.load(data_file)
        for key, value in data.items():
            if key == 'engines' and 'node' in value:
                value_detected = value['node']
                non_decimal = re.compile(r'[^\d.]+')
                # remove the string ~ or  > that sometimes exists in version value
                c = non_decimal.sub('', value_detected)
                # reduce the version to '6.0' from '6.0.0'
                if '.' in c:  # handle version set as 4 instead of 4.0
                    num_array = c.split('.')
                    num = num_array[0] + "." + num_array[1]
                else:
                    num = c + ".0"
                version_detected.append(num)
    return version_detected or ['0.0']


def detect_netcore_version_tocreate(detected_ver):
    if detected_ver in NETCORE_VERSIONS:
        return detected_ver
    return NETCORE_VERSION_DEFAULT


def detect_dotnet_version_tocreate(detected_ver):
    min_ver = DOTNET_VERSIONS[0]
    if detected_ver in DOTNET_VERSIONS:
        return detected_ver
    if detected_ver < min_ver:
        return min_ver
    return ASPDOTNET_VERSION_DEFAULT


def detect_node_version_tocreate(detected_ver):
    if detected_ver in NODE_VERSIONS:
        return detected_ver
    # get major version & get the closest version from supported list
    major_ver = int(detected_ver.split('.')[0])
    node_ver = NODE_VERSION_DEFAULT
    # TODO: Handle checking for minor versions if node major version is 10
    if major_ver <= 11:
        node_ver = NODE_VERSION_DEFAULT
    else:
        node_ver = NODE_VERSION_NEWER
    return node_ver


def find_key_in_json(json_data, key):
    for k, v in json_data.items():
        if key in k:
            yield v
        elif isinstance(v, dict):
            for id_val in find_key_in_json(v, key):
                yield id_val


def set_location(cmd, sku, location):
    client = web_client_factory(cmd.cli_ctx)
    if location is None:
        locs = client.list_geo_regions(sku, True)
        available_locs = []
        for loc in locs:
            available_locs.append(loc.name)
        loc = available_locs[0]
    else:
        loc = location
    return loc.replace(" ", "").lower()


# check if the RG value to use already exists and follows the OS requirements or new RG to be created
def should_create_new_rg(cmd, rg_name, is_linux):
    if (_check_resource_group_exists(cmd, rg_name) and
            _check_resource_group_supports_os(cmd, rg_name, is_linux)):
        return False
    return True


def get_site_availability(cmd, name):
    """ This is used by az webapp up to verify if a site needs to be created or should just be deployed"""
    client = web_client_factory(cmd.cli_ctx)
    return client.check_name_availability(name, 'Microsoft.Web/sites')


def get_app_details(cmd, name):
    client = web_client_factory(cmd.cli_ctx)
    data = (list(filter(lambda x: name.lower() == x.name.lower(), client.web_apps.list())))
    _num_items = len(data)
    if _num_items > 0:
        return data[0]
    return None


def get_rg_to_use(cmd, user, loc, os_name, rg_name=None):
    default_rg = "{}_rg_{}_{}".format(user, os_name, loc.replace(" ", "").lower())
    # check if RG exists & can be used
    if rg_name is not None and _check_resource_group_exists(cmd, rg_name):
        if _check_resource_group_supports_os(cmd, rg_name, os_name.lower() == 'linux'):
            return rg_name
        raise CLIError("The ResourceGroup '{}' cannot be used with the os '{}'. Use a different RG".format(rg_name,
                                                                                                           os_name))
    if rg_name is None:
        rg_name = default_rg
    return rg_name


def get_profile_username():
    from azure.cli.core._profile import Profile
    user = Profile().get_current_account_user()
    user = user.split('@', 1)[0]
    if len(user.split('#', 1)) > 1:  # on cloudShell user is in format live.com#user@domain.com
        user = user.split('#', 1)[1]
    return user


def get_sku_to_use(src_dir, html=False, sku=None, runtime=None):
    if sku is None:
        if runtime:  # user overrided language detection by specifiying runtime
            return 'F1'
        lang_details = get_lang_from_content(src_dir, html)
        return lang_details.get("default_sku")
    logger.info("Found sku argument, skipping use default sku")
    return sku


def set_language(src_dir, html=False):
    lang_details = get_lang_from_content(src_dir, html)
    return lang_details.get('language')


def detect_os_form_src(src_dir, html=False):
    lang_details = get_lang_from_content(src_dir, html)
    language = lang_details.get('language')
    return "Linux" if language is not None and language.lower() == NODE_RUNTIME_NAME \
        or language.lower() == PYTHON_RUNTIME_NAME else OS_DEFAULT


def get_plan_to_use(cmd, user, os_name, loc, sku, create_rg, resource_group_name, plan=None):
    _default_asp = "{}_asp_{}_{}_0".format(user, os_name, loc)
    if plan is None:  # --plan not provided by user
        # get the plan name to use
        return _determine_if_default_plan_to_use(cmd, _default_asp, resource_group_name, loc, sku, create_rg)
    return plan


# Portal uses the current_stack property in the app metadata to display the correct stack
# This value should be one of: ['dotnet', 'dotnetcore', 'node', 'php', 'python', 'java']
def get_current_stack_from_runtime(runtime):
    language = runtime.split('|')[0].lower()
    if language == 'aspnet':
        return 'dotnet'
    return language


# if plan name not provided we need to get a plan name based on the OS, location & SKU
def _determine_if_default_plan_to_use(cmd, plan_name, resource_group_name, loc, sku, create_rg):
    client = web_client_factory(cmd.cli_ctx)
    if create_rg:  # if new RG needs to be created use the default name
        return plan_name
    # get all ASPs in the RG & filter to the ones that contain the plan_name
    _asp_generic = plan_name[:-len(plan_name.split("_")[4])]
    _asp_list = (list(filter(lambda x: _asp_generic in x.name,
                             client.app_service_plans.list_by_resource_group(resource_group_name))))
    _num_asp = len(_asp_list)
    if _num_asp:
        # check if we have at least one app that can be used with the combination of loc, sku & os
        selected_asp = next((a for a in _asp_list if isinstance(a.sku, SkuDescription) and
                             a.sku.name.lower() == sku.lower() and
                             (a.location.replace(" ", "").lower() == loc.lower())), None)
        if selected_asp is not None:
            return selected_asp.name
        # from the sorted data pick the last one & check if a new ASP needs to be created
        # based on SKU or not
        data_sorted = sorted(_asp_list, key=lambda x: x.name)
        _plan_info = data_sorted[_num_asp - 1]
        _asp_num = 1
        try:
            _asp_num = int(_plan_info.name.split('_')[-1]) + 1  # default asp created by CLI can be of type plan_num
        except ValueError:
            pass
        return '{}_{}'.format(_asp_generic, _asp_num)
    return plan_name


def should_create_new_app(cmd, rg_name, app_name):  # this is currently referenced by an extension command
    client = web_client_factory(cmd.cli_ctx)
    for item in list(client.web_apps.list_by_resource_group(rg_name)):
        if item.name.lower() == app_name.lower():
            return False
    return True


def generate_default_app_name(cmd):
    def generate_name(cmd):
        import uuid
        from random import choice

        noun = choice(get_file_json(GENERATE_RANDOM_APP_NAMES)['APP_NAME_NOUNS'])
        adjective = choice(get_file_json(GENERATE_RANDOM_APP_NAMES)['APP_NAME_ADJECTIVES'])
        random_uuid = str(uuid.uuid4().hex)

        name = '{}-{}-{}'.format(adjective, noun, random_uuid)
        name_available = get_site_availability(cmd, name).name_available

        if name_available:
            return name
        return ""

    retry_times = 5
    generated_name = generate_name(cmd)
    while not generated_name and retry_times > 0:
        retry_times -= 1
        generated_name = generate_name(cmd)

    if not generated_name:
        raise CLIError("Unable to generate a default name for webapp. Please specify webapp name using --name flag")
    return generated_name



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import inspect
from importlib import import_module
from pkgutil import iter_modules
import yaml
import colorama
from .util import share_element, exclude_commands, LinterError


class Linter(object):
    def __init__(self, command_loader=None, help_file_entries=None, loaded_help=None):
        self._all_yaml_help = help_file_entries
        self._loaded_help = loaded_help
        self._command_loader = command_loader
        self._parameters = {}
        self._help_file_entries = set(help_file_entries.keys())
        self._command_parser = command_loader.cli_ctx.invocation.parser
        for command_name, command in self._command_loader.command_table.items():
            self._parameters[command_name] = set()
            for name, param in command.arguments.items():
                self._parameters[command_name].add(name)

    @property
    def commands(self):
        return self._command_loader.command_table.keys()

    @property
    def command_groups(self):
        return self._command_loader.command_group_table.keys()

    @property
    def help_file_entries(self):
        return self._help_file_entries

    @property
    def command_parser(self):
        return self._command_parser

    def get_command_metadata(self, command_name):
        try:
            return self._command_loader.command_table[command_name]
        except KeyError:
            return None

    def get_command_parameters(self, command_name):
        return self._parameters.get(command_name)

    def get_command_group_metadata(self, command_group_name):
        try:
            return self._command_loader.command_group_table[command_group_name]
        except KeyError:
            return None

    def get_help_entry_type(self, entry_name):
        return self._all_yaml_help.get(entry_name).get('type')

    def get_help_entry_examples(self, entry_name):
        return self._all_yaml_help.get(entry_name).get('examples', [])

    def get_help_entry_parameter_names(self, entry_name):
        return [param_help.get('name', None) for param_help in \
            self._all_yaml_help.get(entry_name).get('parameters', [])]

    def is_valid_parameter_help_name(self, entry_name, param_name):
        return param_name in [param.name for param in getattr(self._loaded_help.get(entry_name), 'parameters', [])]

    def get_command_help(self, command_name):
        return self._get_loaded_help_description(command_name)

    def get_command_group_help(self, command_group_name):
        return self._get_loaded_help_description(command_group_name)

    def get_parameter_options(self, command_name, parameter_name):
        return self.get_command_metadata(command_name).arguments.get(parameter_name).type.settings.get('options_list')

    def get_parameter_help(self, command_name, parameter_name):
        options = self.get_parameter_options(command_name, parameter_name)
        command_help = self._loaded_help.get(command_name, None)

        if not command_help:
            return None

        parameter_helps = command_help.parameters
        param_help = next((param for param in parameter_helps if share_element(options, param.name.split())), None)
        # workaround for --ids which is not does not generate doc help (BUG)
        if not param_help:
            return self._command_loader.command_table.get(command_name).arguments.get(parameter_name).type.settings.get('help')
        return param_help.short_summary or param_help.long_summary

    def command_expired(self, command_name):
        deprecate_info = self._command_loader.command_table[command_name].deprecate_info
        if deprecate_info:
            return deprecate_info.expired()
        return False

    def command_group_expired(self, command_group_name):
        try:
            deprecate_info = self._command_loader.command_group_table[command_group_name].group_kwargs.get('deprecate_info', None)
            if deprecate_info:
                return deprecate_info.expired()
        except AttributeError:
            # Items with only token presence in the command table will not have any data. They can't be expired.
            pass
        return False

    def parameter_expired(self, command_name, parameter_name):
        parameter = self._command_loader.command_table[command_name].arguments[parameter_name].type.settings
        deprecate_info = parameter.get('deprecate_info', None)
        if deprecate_info:
            return deprecate_info.expired()
        return False

    def option_expired(self, command_name, parameter_name):
        from knack.deprecation import Deprecated
        parameter = self._command_loader.command_table[command_name].arguments[parameter_name].type.settings
        options_list = parameter.get('options_list', [])
        expired_options_list = []
        for opt in options_list:
            if isinstance(opt, Deprecated) and opt.expired():
                expired_options_list.append(opt.target)
        return expired_options_list

    def _get_loaded_help_description(self, entry):
        help_entry = self._loaded_help.get(entry, None)
        if help_entry:
            return help_entry.short_summary or help_entry.long_summary
        return help_entry


class LinterManager(object):
    def __init__(self, command_loader=None, help_file_entries=None, loaded_help=None, exclusions=None,
                 rule_inclusions=None):
        self.linter = Linter(command_loader=command_loader, help_file_entries=help_file_entries, loaded_help=loaded_help)
        self._exclusions = exclusions or {}
        self._rules = {
            'help_file_entries': {},
            'command_groups': {},
            'commands': {},
            'params': {}
        }
        self._ci_exclusions = {}
        self._rule_inclusions = rule_inclusions
        self._loaded_help = loaded_help
        self._command_loader = command_loader
        self._help_file_entries = help_file_entries
        self._exit_code = 0
        self._ci = False

    def add_rule(self, rule_type, rule_name, rule_callable):
        include_rule = not self._rule_inclusions or rule_name in self._rule_inclusions
        if rule_type in self._rules and include_rule:
            def get_linter():
                if rule_name in self._ci_exclusions and self._ci:
                    mod_exclusions = self._ci_exclusions[rule_name]
                    command_loader, help_file_entries = exclude_commands(self._command_loader, self._help_file_entries,
                                                                        mod_exclusions)
                    return Linter(command_loader=command_loader, help_file_entries=help_file_entries,
                                  loaded_help=self._loaded_help)
                return self.linter
            self._rules[rule_type][rule_name] = rule_callable, get_linter

    def mark_rule_failure(self):
        self._exit_code = 1

    @property
    def exclusions(self):
        return self._exclusions

    @property
    def exit_code(self):
        return self._exit_code

    def run(self, run_params=None, run_commands=None, run_command_groups=None, run_help_files_entries=None, ci=False):
        self._ci = ci
        paths = import_module('automation.cli_linter.rules').__path__

        if paths:
            ci_exclusions_path = os.path.join(paths[0], 'ci_exclusions.yml')
            self._ci_exclusions = yaml.safe_load(open(ci_exclusions_path)) or {}

        # find all defined rules and check for name conflicts
        found_rules = set()
        for _, name, _ in iter_modules(paths):
            rule_module = import_module('automation.cli_linter.rules.' + name)
            functions = inspect.getmembers(rule_module, inspect.isfunction)
            for rule_name, add_to_linter_func in functions:
                if hasattr(add_to_linter_func, 'linter_rule'):
                    if rule_name in found_rules:
                        raise LinterError('Multiple rules found with the same name: %s' % rule_name)
                    found_rules.add(rule_name)
                    add_to_linter_func(self)

        colorama.init()
        # run all rule-checks
        if run_help_files_entries and self._rules.get('help_file_entries'):
            self._run_rules('help_file_entries')

        if run_command_groups and self._rules.get('command_groups'):
            self._run_rules('command_groups')

        if run_commands and self._rules.get('commands'):
            self._run_rules('commands')

        if run_params and self._rules.get('params'):
            self._run_rules('params')

        if not self.exit_code:
            print(os.linesep + 'No violations found.')
        colorama.deinit()
        return self.exit_code

    def _run_rules(self, rule_group):
        from colorama import Fore
        for rule_name, (rule_func, linter_callable) in self._rules.get(rule_group).items():
            # use new linter if needed
            with LinterScope(self, linter_callable):
                violations = sorted(rule_func()) or []
                if violations:
                    print('- {} FAIL{}: {}'.format(Fore.RED, Fore.RESET, rule_name))
                    for violation_msg in violations:
                        print(violation_msg)
                    print()
                else:
                    print('- {} pass{}: {} '.format(Fore.GREEN, Fore.RESET, rule_name))


class RuleError(Exception):
    """
    Exception thrown by rule violation
    """
    pass


class LinterScope():
    def __init__(self, linter_manager, linter_callable):
        self.linter_manager = linter_manager
        self.linter = linter_callable()
        self.main_linter = linter_manager.linter

    def __enter__(self):
        self.linter_manager.linter = self.linter

    def __exit__(self, exc_type, value, traceback):
        self.linter_manager.linter = self.main_linter



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer, api_version_constraint,
                               JMESPathCheck, JMESPathCheckExists)
from azure.cli.core.profiles import ResourceType
from ..storage_test_util import StorageScenarioMixin
from azure_devtools.scenario_tests import AllowLargeResponse


class StorageFileShareRmScenarios(StorageScenarioMixin, ScenarioTest):
    @AllowLargeResponse()
    @api_version_constraint(ResourceType.MGMT_STORAGE, min_api='2019-06-01')
    @ResourceGroupPreparer(name_prefix="cli", location="eastus")
    @StorageAccountPreparer(name_prefix="sharerm", location="eastus")
    def test_storage_file_using_rm_main_scenario(self):
        # 1. Test create command.

        # Create file share with storage account name and resource group.
        share_name_1 = self.create_random_name('share', 24)
        initial_quota = 5
        self.kwargs.update({
            'share_name_1': share_name_1,
            'initial_quota': initial_quota
        })

        result = self.cmd('storage share-rm create --storage-account {sa} -g {rg} -n {share_name_1} --quota {initial_quota} --metadata key1=value1').get_output_in_json()
        self.assertEqual(result['name'], share_name_1)
        self.assertEqual(result['shareQuota'], initial_quota)
        self.assertEqual(result['metadata']['key1'], 'value1')

        share_id_1 = result['id']
        self.kwargs.update({
            'share_id_1': share_id_1
        })

        # Create file share with storage account id.
        share_name_2 = self.create_random_name('share', 24)
        storage_account = self.cmd('storage account show -n {sa}').get_output_in_json()
        storage_account_id = storage_account['id']
        self.kwargs.update({
            'share_name_2': share_name_2,
            'storage_account_id': storage_account_id
        })

        result = self.cmd('storage share-rm create --storage-account {storage_account_id} -n {share_name_2} --quota {initial_quota} --metadata key1=value1').get_output_in_json()
        self.assertEqual(result['name'], share_name_2)
        self.assertEqual(result['shareQuota'], initial_quota)
        self.assertEqual(result['metadata']['key1'], 'value1')

        share_id_2 = result['id']
        self.kwargs.update({
            'share_id_2': share_id_2
        })

        # 2. Test exists command (the file share exists).

        # Check existence with storage account name and resource group.
        result = self.cmd('storage share-rm exists --storage-account {sa} -g {rg} -n {share_name_1}').get_output_in_json()
        self.assertEqual(result['exists'], True)

        # Check existence with storage account id.
        result = self.cmd('storage share-rm exists --storage-account {storage_account_id} -n {share_name_1}').get_output_in_json()
        self.assertEqual(result['exists'], True)

        # Check existence by file share resource id.
        result = self.cmd('storage share-rm exists --ids {share_id_1}').get_output_in_json()
        self.assertEqual(result['exists'], True)

        # 3. Test show command (the file share exists).

        # Show properties of a file share with storage account name and resource group.
        result = self.cmd('storage share-rm show --storage-account {sa} -g {rg} -n {share_name_1}').get_output_in_json()
        self.assertEqual(result['name'], share_name_1)

        # Show properties of a file share with storage account id.
        result = self.cmd('storage share-rm show --storage-account {storage_account_id} -n {share_name_1}').get_output_in_json()
        self.assertEqual(result['name'], share_name_1)

        # Show properties by file share resource id.
        result = self.cmd('storage share-rm show --ids {share_id_1}').get_output_in_json()
        self.assertEqual(result['name'], share_name_1)

        # 4. Test show command (the file share doesn't exist).
        non_exist_share_name = self.create_random_name('share', 24)
        self.kwargs.update({
            'non_exist_share_name': non_exist_share_name
        })
        with self.assertRaisesRegexp(SystemExit, '3'):
            self.cmd('storage share-rm show --storage-account {sa} -g {rg} -n {non_exist_share_name}')

        # 5. Test update command.
        updated_quota = 10
        self.kwargs.update({
            'updated_quota': updated_quota
        })

        # Update file share with storage account name and resource group.
        result = self.cmd(
            'storage share-rm update --storage-account {sa} -g {rg} -n {share_name_1} --quota {updated_quota} --metadata key2=value2').get_output_in_json()
        self.assertEqual(result['shareQuota'], updated_quota)
        self.assertEqual(result['metadata']['key2'], 'value2')
        self.assertNotIn('key1', result['metadata'])

        # Update file share with storage account id.
        result = self.cmd(
            'storage share-rm update --storage-account {storage_account_id} -n {share_name_2} --quota {updated_quota} --metadata key2=value2').get_output_in_json()
        self.assertEqual(result['shareQuota'], updated_quota)
        self.assertEqual(result['metadata']['key2'], 'value2')
        self.assertNotIn('key1', result['metadata'])

        # Update file share by resource id
        result = self.cmd(
            'storage share-rm update --ids {share_id_1} --quota {updated_quota} --metadata key2=value2').get_output_in_json()
        self.assertEqual(result['shareQuota'], updated_quota)
        self.assertEqual(result['metadata']['key2'], 'value2')
        self.assertNotIn('key1', result['metadata'])

        # 6. Test list command.

        # List file shares with storage account name and resource group.
        self.assertIn(share_name_1,
                      self.cmd('storage share-rm list --storage-account {sa} -g {rg} --query "[].name"').get_output_in_json())

        # List file shares with storage account id.
        self.assertIn(share_name_1,
                      self.cmd('storage share-rm list --storage-account {storage_account_id} --query "[].name"').get_output_in_json())

        # 7. Test delete command.

        # Delete file shares with storage account name and resource group.
        self.cmd('storage share-rm delete --storage-account {sa} -g {rg} -n {share_name_1} -y')

        # Delete file share by resource id.
        self.cmd('storage share-rm delete --ids {share_id_2} -y')

        # 8. Test exists command (the file share doesn't exist).
        result = self.cmd('storage share-rm exists --storage-account {sa} -g {rg} -n {share_name_1}').get_output_in_json()
        self.assertEqual(result['exists'], False)

        result = self.cmd('storage share-rm exists --ids {share_id_2}').get_output_in_json()
        self.assertEqual(result['exists'], False)

    @api_version_constraint(ResourceType.MGMT_STORAGE, min_api='2019-06-01')
    @ResourceGroupPreparer(name_prefix="cli_nfs", location="eastus2euap")
    @StorageAccountPreparer(name_prefix="nfs", location="eastus2", kind='FileStorage', sku='Premium_LRS')
    def test_storage_share_rm_with_NFS(self):

        self.kwargs.update({
            'share': self.create_random_name('share', 24),
        })
        self.cmd('storage share-rm create --storage-account {sa} -g {rg} -n {share} --enabled-protocols NFS --root-squash AllSquash', checks={
            JMESPathCheck('name', self.kwargs['share']),
            JMESPathCheck('enabledProtocol', 'NFS'),
            JMESPathCheck('rootSquash', 'AllSquash')
        })

        self.cmd('storage share-rm update --storage-account {sa} -g {rg} -n {share} --root-squash NoRootSquash', checks={
            JMESPathCheck('name', self.kwargs['share']),
            JMESPathCheck('rootSquash', 'NoRootSquash')
        })

        self.cmd('storage share-rm show --storage-account {sa} -g {rg} -n {share}', checks={
            JMESPathCheck('name', self.kwargs['share']),
            JMESPathCheck('enabledProtocol', 'NFS'),
            JMESPathCheck('rootSquash', 'NoRootSquash')
        })

        self.cmd('storage share-rm list --storage-account {sa} -g {rg}', checks={
            JMESPathCheck('[0].name', self.kwargs['share']),
            JMESPathCheck('[0].enabledProtocol', 'NFS'),
            JMESPathCheck('[0].rootSquash', 'NoRootSquash')
        })

        self.cmd('storage share-rm delete --storage-account {sa} -g {rg} -n {share} -y')
        self.cmd('storage share-rm list --storage-account {sa} -g {rg}', checks={
            JMESPathCheck('length(@)', 0)
        })

    @api_version_constraint(ResourceType.MGMT_STORAGE, min_api='2019-06-01')
    @ResourceGroupPreparer(name_prefix="cli_tier", location="eastus")
    @StorageAccountPreparer(name_prefix="tier", location="eastus", kind='StorageV2')
    def test_storage_share_rm_with_access_tier(self):

        self.kwargs.update({
            'share': self.create_random_name('share', 24),
            'new_share': self.create_random_name('share', 24)
        })

        # Update with access tier
        self.cmd('storage share-rm create --storage-account {sa} -g {rg} -n {share}', checks={
            JMESPathCheck('name', self.kwargs['share']),
            JMESPathCheck('accessTier', None)
        })

        self.cmd('storage share-rm show --storage-account {sa} -g {rg} -n {share}', checks={
            JMESPathCheck('name', self.kwargs['share']),
            JMESPathCheck('accessTier', 'TransactionOptimized')
        })

        self.cmd('storage share-rm update --storage-account {sa} -g {rg} -n {share} --access-tier Hot', checks={
            JMESPathCheck('name', self.kwargs['share']),
            JMESPathCheck('accessTier', 'Hot')
        })

        self.cmd('storage share-rm show --storage-account {sa} -g {rg} -n {share}', checks={
            JMESPathCheck('name', self.kwargs['share']),
            JMESPathCheck('accessTier', 'Hot'),
            JMESPathCheckExists('accessTierChangeTime')
        })

        # Create with access tier
        self.cmd('storage share-rm create --storage-account {sa} -g {rg} -n {new_share} --access-tier Hot', checks={
            JMESPathCheck('[0].name', self.kwargs['share']),
            JMESPathCheck('accessTier', 'Hot')
        })

    @AllowLargeResponse()
    @ResourceGroupPreparer(name_prefix="cli_stats")
    @StorageAccountPreparer(name_prefix="stats")
    def test_storage_share_rm_with_stats(self, resource_group, storage_account):
        account_info = self.get_account_info(group=resource_group, name=storage_account)
        share_name = self.create_random_name(prefix='share', length=12)
        self.cmd('storage share-rm create -n {} --storage-account {} -g {} '.format(
            share_name, storage_account, resource_group))
        result = self.cmd('storage share-rm stats -n {} --storage-account {} -g {} '.format(
            share_name, storage_account, resource_group)).output.strip('\n')
        self.assertEqual(result, str(0))

        local_file = self.create_temp_file(512, full_random=False)

        self.storage_cmd('storage file upload --share-name {} --source "{}" --path source_file.txt ', account_info,
                         share_name, local_file)
        result = self.cmd('storage share-rm stats -n {} --storage-account {} -g {} '.format(
            share_name, storage_account, resource_group)).output.strip('\n')
        self.assertEqual(result, str(512 * 1024))

    @api_version_constraint(ResourceType.MGMT_STORAGE, min_api='2019-06-01')
    @ResourceGroupPreparer(name_prefix="cli_sf", location="francecentral")
    @StorageAccountPreparer(name_prefix="clitest", location="francecentral")
    def test_storage_share_rm_soft_delete(self):
        import time
        self.cmd('storage account file-service-properties update -n {sa} -g {rg} --delete-retention-days 7 --enable-delete-retention',
                 checks={
                     JMESPathCheck('shareDeleteRetentionPolicy.days', 7),
                     JMESPathCheck('shareDeleteRetentionPolicy.enabled', True)
                 })

        self.kwargs.update({
            'share': self.create_random_name(prefix='share', length=24)
        })
        self.cmd('storage share-rm create --storage-account {sa} -g {rg} -n {share}', checks={
            JMESPathCheck('name', self.kwargs['share'])
        })

        self.cmd('storage share-rm list --storage-account {sa} -g {rg}', checks={
            JMESPathCheck('length(@)', 1)
        })
        self.cmd('storage share-rm delete --storage-account {sa} -g {rg} -n {share} -y')
        self.cmd('storage share-rm list --storage-account {sa} -g {rg}', checks={
            JMESPathCheck('length(@)', 0)
        })
        self.cmd('storage share-rm list --storage-account {sa} -g {rg} --include-deleted', checks={
            JMESPathCheck('length(@)', 1),
            JMESPathCheck('[0].deleted', True)
        })

        time.sleep(30)
        self.kwargs['version'] = \
            self.cmd('storage share-rm list --storage-account {sa} -g {rg} --include-deleted --query [0].version -o tsv'
                     ).output.strip('\n')
        self.cmd('storage share-rm restore --storage-account {sa} -g {rg} -n {share} --deleted-version {version}',
                 checks={
                     JMESPathCheck('name', self.kwargs['share'])})

        self.cmd('storage share-rm list --storage-account {sa} -g {rg}', checks={
            JMESPathCheck('length(@)', 1)
        })



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest


class TestStyle(unittest.TestCase):

    def test_format_styled_text(self):
        from azure.cli.core.style import Style, format_styled_text
        styled_text = [
            (Style.PRIMARY, "Bright White: Primary text color\n"),
            (Style.SECONDARY, "White: Secondary text color\n"),
            (Style.IMPORTANT, "Bright Magenta: Important text color\n"),
            (Style.ACTION, "Bright Blue: Commands, parameters, and system inputs\n"),
            (Style.HYPERLINK, "Bright Cyan: Hyperlink\n"),
            (Style.ERROR, "Bright Red: Error message indicator\n"),
            (Style.SUCCESS, "Bright Green: Success message indicator\n"),
            (Style.WARNING, "Bright Yellow: Warning message indicator\n"),
        ]
        formatted = format_styled_text(styled_text)
        excepted = """\x1b[97mBright White: Primary text color
\x1b[90mWhite: Secondary text color
\x1b[95mBright Magenta: Important text color
\x1b[94mBright Blue: Commands, parameters, and system inputs
\x1b[96mBright Cyan: Hyperlink
\x1b[91mBright Red: Error message indicator
\x1b[92mBright Green: Success message indicator
\x1b[93mBright Yellow: Warning message indicator
\x1b[39m"""
        self.assertEqual(formatted, excepted)

        # Test invalid style
        from azure.cli.core.azclierror import CLIInternalError
        with self.assertRaisesRegex(CLIInternalError, "Invalid style."):
            format_styled_text([("invalid_style", "dummy text",)])

        # Test invalid styled style
        with self.assertRaisesRegex(CLIInternalError, "Invalid styled text."):
            format_styled_text([(Style.PRIMARY,)])
        with self.assertRaisesRegex(CLIInternalError, "Invalid styled text."):
            format_styled_text(["dummy text"])


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long
from __future__ import print_function

import datetime
import os
import time
from shutil import rmtree
from msrestazure.azure_exceptions import CloudError

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, LiveScenarioTest, VirtualNetworkPreparer
from azure_devtools.scenario_tests import AllowLargeResponse

from knack.util import CLIError


class DataLakeStoreFileAccessScenarioTest(ScenarioTest):

    @ResourceGroupPreparer(name_prefix='cli_test_adls_access')
    def test_dls_file_access_mgmt(self):

        user_id = '470c0ccf-c91a-4597-98cd-48507d2f1486'

        self.kwargs.update({
            'dls': self.create_random_name('cliadls', 24),
            'loc': 'eastus2',
            'dir': 'adltestfolder01',
            'user_id': user_id,
            'acl_to_add': 'user:{}:rwx'.format(user_id),
            'acl_to_modify': 'user:{}:-w-'.format(user_id),
            'acl_to_remove': 'user:{}'.format(user_id)
        })

        # create ADLS account
        self.cmd('dls account create -g {rg} -n {dls} -l {loc} --disable-encryption')
        result = self.cmd('dls account show -g {rg} -n {dls}').get_output_in_json()
        while result['provisioningState'] != 'Succeeded' and result['provisioningState'] != 'Failed':
            time.sleep(5)
            result = self.cmd('dls account show -g {rg} -n {dls}').get_output_in_json()

        if result['provisioningState'] == 'Failed':
            raise CLIError('Failed to create the adls account, tests cannot proceed!')

        # create a folder
        self.cmd('dls fs create -n {dls} --path "{dir}" --folder --force')
        # get the folder
        self.cmd('dls fs show -n {dls} --path "{dir}"', checks=[
            self.check('name', '{dir}'),
            self.check('type', 'DIRECTORY'),
            self.check('length', 0),
        ])

        # set the owner and owning group for the file and confirm them
        self.kwargs.update({
            'group_id': '80a3ed5f-959e-4696-ba3c-d3c8b2db6766',
            'user_id': '6361e05d-c381-4275-a932-5535806bb323'
        })
        self.cmd('dls fs access set-owner -n {dls} --path "{dir}" --group {group_id} --owner {user_id}')

        # get the file and confirm those values
        self.cmd('dls fs show -n {dls} --path "{dir}"', checks=[
            self.check('name', '{dir}'),
            self.check('type', 'DIRECTORY'),
            self.check('length', 0),
            self.check('owner', '{user_id}'),
            self.check('group', '{group_id}'),
        ])

        # set the permissions on the file
        self.cmd('dls fs access set-permission -n {dls} --path "{dir}" --permission 777')
        # get the file and confirm those values
        self.cmd('dls fs show -n {dls} --path "{dir}"', checks=[
            self.check('name', '{dir}'),
            self.check('type', 'DIRECTORY'),
            self.check('length', 0),
            self.check('permission', '777'),
        ])

        # get the initial ACE
        result = self.cmd('dls fs access show -n {dls} --path "{dir}"').get_output_in_json()
        inital_acl_length = len(result['entries'])
        new_acl = ','.join(result['entries'])
        new_acl += ',{acl_to_add}'.format(**self.kwargs)
        self.kwargs['new_acl'] = new_acl
        # set the full ACL
        self.cmd('dls fs access set -n {dls} --path "{dir}" --acl-spec {new_acl}')
        # get the ACL and confirm that it has grown
        set_result = self.cmd('dls fs access show -n {dls} --path "{dir}"').get_output_in_json()
        assert len(set_result['entries']) > inital_acl_length
        assert self.kwargs['acl_to_add'] in set_result['entries']

        # modify that ACE with set-entry
        self.cmd('dls fs access set-entry -n {dls} --path "{dir}" --acl-spec {acl_to_modify}')
        # get the ACL and confirm it has been modified
        modify_result = self.cmd('dls fs access show -n {dls} --path "{dir}"').get_output_in_json()
        assert len(set_result['entries']) > inital_acl_length
        assert self.kwargs['acl_to_modify'] in modify_result['entries']
        # remove an ACE
        self.cmd('dls fs access remove-entry -n {dls} --path "{dir}" --acl-spec {acl_to_remove}')
        # get ACL and ensure that it is smaller than before
        remove_result = self.cmd('dls fs access show -n {dls} --path "{dir}"').get_output_in_json()
        assert len(remove_result['entries']) < len(modify_result['entries'])
        assert self.kwargs['acl_to_modify'] not in remove_result['entries']
        # remove default ACL
        self.cmd('dls fs access remove-all -n {dls} --path "{dir}" --default-acl')
        remove_result = self.cmd('dls fs access show -n {dls} --path "{dir}"').get_output_in_json()
        # there should be four entries left
        assert 4 == len(remove_result['entries'])

        # remove ACL
        self.cmd('dls fs access remove-all -n {dls} --path "{dir}"')
        remove_result = self.cmd('dls fs access show -n {dls} --path "{dir}"').get_output_in_json()
        # there should be three entries left
        assert 3 == len(remove_result['entries'])


class DataLakeStoreFileScenarioTest(ScenarioTest):

    def setUp(self):
        try:
            import unittest.mock as mock
        except ImportError:
            import mock
        import uuid

        def const_uuid():
            return uuid.UUID('{12345678-1234-5678-1234-567812345678}')

        self.mp = mock.patch('uuid.uuid4', const_uuid)
        self.mp.__enter__()
        super(DataLakeStoreFileScenarioTest, self).setUp()

    def tearDown(self):
        local_folder = self.kwargs.get('local_folder', None)
        if local_folder and os.path.exists(local_folder):
            rmtree(local_folder)
        self.mp.__exit__(None, None, None)
        return super(DataLakeStoreFileScenarioTest, self).tearDown()

    @ResourceGroupPreparer(name_prefix='cls_test_adls_file')
    def test_dls_file_mgmt(self):

        local_folder = 'adls_resources'
        self.kwargs.update({
            'dls': self.create_random_name('cliadls', 24),
            'loc': 'eastus2',
            'dir': local_folder,
            'local_folder': os.path.join(os.getcwd(), local_folder),
            'local_file': os.path.join(local_folder, 'sample_file.txt'),
            'local_file_content': 'Local File Content',
            'folder1': 'adltestfolder01',
            'folder2': 'adltestfolder02',
            'file': 'adltestfile01',
            'upload_file': 'adltestfile02',
            'join_file': 'adltestfile03',
            'download_file': 'adltestfile04',
            'content': '123456'
        })

        # create ADLS account
        self.cmd('dls account create -g {rg} -n {dls} -l {loc} --disable-encryption')
        result = self.cmd('dls account show -g {rg} -n {dls}').get_output_in_json()
        while result['provisioningState'] != 'Succeeded' and result['provisioningState'] != 'Failed':
            time.sleep(5)
            result = self.cmd('dls account show -g {rg} -n {dls}').get_output_in_json()

        if result['provisioningState'] == 'Failed':
            raise CLIError('Failed to create the adls account, tests cannot proceed!')

        # create local file
        if os.path.exists(local_folder):
            rmtree(local_folder)

        os.makedirs(local_folder)
        with open(self.kwargs['local_file'], 'w') as f:
            f.write(self.kwargs['local_file_content'])

        # file and folder manipulation
        # create a folder
        self.cmd('dls fs create -n {dls} --path "{folder1}" --folder --force')
        # get the folder
        self.cmd('dls fs show -n {dls} --path "{folder1}"', checks=[
            self.check('name', '{folder1}'),
            self.check('type', 'DIRECTORY'),
            self.check('length', 0),
        ])

        # create a file
        self.cmd('dls fs create -n {dls} --path "{folder1}/{file}" --content {content}')
        # get the file
        result = self.cmd('dls fs show -n {dls} --path "{folder1}/{file}"', checks=[
            self.check('name', '{folder1}/{file}'),
            self.check('type', 'FILE'),
            self.check('length', len(self.kwargs['content'])),
        ]).get_output_in_json()

        # set expiration time on the file
        # this future time gives the milliseconds since the epoch that have occured as of 01/31/2030 at noon
        epoch_time = datetime.datetime.utcfromtimestamp(0)
        final_time = datetime.datetime(2030, 1, 31, 12)
        self.kwargs['time_in_milliseconds'] = (final_time - epoch_time).total_seconds() * 1000
        self.cmd('dls fs set-expiry -n {dls} --path "{folder1}/{file}" --expiration-time {time_in_milliseconds}')
        new_result = self.cmd('dls fs show -n {dls} --path "{folder1}/{file}"', checks=[
            self.check('name', '{folder1}/{file}'),
            self.check('type', 'FILE'),
            self.check('length', len(self.kwargs['content'])),
        ]).get_output_in_json()

        # this value is not guaranteed to be exact, but it should be within a very small range (around 200ms)
        assert self.kwargs['time_in_milliseconds'] - 100 < new_result['msExpirationTime'] < self.kwargs['time_in_milliseconds'] + 100

        # remove the expiration time
        self.cmd('dls fs remove-expiry -n {dls} --path "{folder1}/{file}"')
        self.cmd('dls fs show -n {dls} --path "{folder1}/{file}"', checks=[
            self.check('name', '{folder1}/{file}'),
            self.check('type', 'FILE'),
            self.check('length', len(self.kwargs['content'])),
            self.check('msExpirationTime', result['msExpirationTime']),
        ])

        # move the file
        # this requires that the folder exists
        self.cmd('dls fs create -n {dls} --path "{folder2}" --folder --force')
        self.cmd('dls fs move -n {dls} --source-path "{folder1}/{file}" --destination-path "{folder2}/{file}"')
        # get the file at the new location
        self.cmd('dls fs show -n {dls} --path "{folder2}/{file}"', checks=[
            self.check('name', '{folder2}/{file}'),
            self.check('type', 'FILE'),
            self.check('length', len(self.kwargs['content'])),
        ])

        # preview the file
        result = self.cmd('dls fs preview -n {dls} --path "{folder2}/{file}"').get_output_in_json()
        assert result == self.kwargs['content']

        # partial file preview
        result = self.cmd('dls fs preview -n {dls} --path "{folder2}/{file}" --length 1 --offset 3').get_output_in_json()
        assert len(result) == 1
        assert result == '4'

        # list the directory, which contains just the one file
        self.cmd('dls fs list -n {dls} --path "{folder2}"', checks=[
            self.check('type(@)', 'array'),
            self.check('length(@)', 1),
            self.check('[0].pathSuffix', '{file}'),
            self.check('[0].type', 'FILE'),
            self.check('[0].length', len(self.kwargs['content'])),
        ])

        # append content to a file
        self.cmd('dls fs append -n {dls} --path "{folder2}/{file}" --content {content}')
        # get the file
        self.cmd('dls fs show -n {dls} --path "{folder2}/{file}"', checks=[
            self.check('name', '{folder2}/{file}'),
            self.check('type', 'FILE'),
            self.check('length', len(self.kwargs['content']) * 2),
        ])

        # upload a file
        self.cmd('dls fs upload -n {dls} --destination-path "{folder1}/{upload_file}" --source-path "{local_file}"')
        # get the file
        self.cmd('dls fs show -n {dls} --path "{folder1}/{upload_file}"', checks=[
            self.check('name', '{folder1}/{upload_file}'),
            self.check('type', 'FILE'),
            self.check('length', len(self.kwargs['local_file_content'])),
        ])
        # join the uploaded file to the created file
        self.cmd('dls fs join -n {dls} --destination-path "{folder1}/{join_file}" --source-paths "{folder1}/{upload_file}" "{folder2}/{file}"')
        self.cmd('dls fs show -n {dls} --path "{folder1}/{join_file}"', checks=[
            self.check('name', '{folder1}/{join_file}'),
            self.check('type', 'FILE'),
            self.check('length', len(self.kwargs['local_file_content']) + (len(self.kwargs['content']) * 2)),
        ])

        # download the joined file
        self.cmd('dls fs download -n {dls} --destination-path "{dir}/{download_file}" --source-path "{folder1}/{join_file}"')
        assert os.path.getsize(os.path.join(self.kwargs['dir'], self.kwargs['download_file'])) == len(self.kwargs['local_file_content']) + (len(self.kwargs['content']) * 2)

        # delete the file and confirm it is gone.
        self.cmd('dls fs delete -n {dls} --path "{folder1}/{join_file}"')
        self.cmd('dls fs list -n {dls} --path "{folder1}"', checks=[
            self.check('type(@)', 'array'),
            self.check('length(@)', 0),
        ])

        # delete a folder that has contents and confirm it is gone.
        self.cmd('dls fs create -n {dls} --path "{folder2}" --folder')
        self.cmd('dls fs create -n {dls} --path "{folder2}/tempfile01.txt"')
        self.cmd('dls fs delete -n {dls} --path "{folder2}" --recurse')
        time.sleep(10)

        # test that the path is gone
        result = self.cmd('dls fs test -n {dls} --path "{folder2}"').get_output_in_json()
        self.assertTrue(not result)

        # test that the other folder still exists
        result = self.cmd('dls fs test -n {dls} --path "{folder1}"').get_output_in_json()
        self.assertTrue(result)


class DataLakeStoreAccountScenarioTest(ScenarioTest):

    @ResourceGroupPreparer(name_prefix='cli_test_adls_mgmt')
    @VirtualNetworkPreparer()
    @AllowLargeResponse()
    def test_dls_account_mgmt(self, resource_group):

        self.kwargs.update({
            'dls': self.create_random_name('cliadls', 24),
            'loc': 'eastus2',
            'updated_subnet': 'updatedSubnet'
        })

        # test create keyvault with default access policy set
        self.cmd('dls account create -g {rg} -n {dls} -l {loc}', checks=[
            self.check('name', '{dls}'),
            self.check('location', '{loc}'),
            self.check('resourceGroup', '{rg}'),
            self.check('encryptionState', 'Enabled'),
        ])
        self.cmd('dls account show -n {dls} -g {rg}', checks=[
            self.check('name', '{dls}'),
            self.check('location', '{loc}'),
            self.check('resourceGroup', '{rg}'),
            self.check('encryptionState', 'Enabled'),
        ])

        # attempt to enable the key vault when it is already enabled, which should throw
        with self.assertRaises(CloudError):
            self.cmd('dls account enable-key-vault -n {dls} -g {rg}')

        self.cmd('dls account list -g {rg}', checks=[
            self.check('type(@)', 'array'),
            self.check('length(@)', 1),
            self.check('[0].name', '{dls}'),
            self.check('[0].location', '{loc}'),
            self.check('[0].resourceGroup', '{rg}'),
        ])
        result = self.cmd('dls account list').get_output_in_json()
        assert isinstance(result, list)
        assert len(result) >= 1

        # test update acct
        self.cmd('dls account update -g {rg} -n {dls} --firewall-state Enabled --trusted-id-provider-state Enabled')
        self.cmd('dls account show -n {dls} -g {rg}', checks=[
            self.check('name', '{dls}'),
            self.check('location', '{loc}'),
            self.check('resourceGroup', '{rg}'),
            self.check('firewallState', 'Enabled'),
            self.check('trustedIdProviderState', 'Enabled'),
        ])

        # test firewall crud
        self.kwargs.update({
            'fw': 'testfirewallrule01',
            'start_ip': '127.0.0.1',
            'end_ip': '127.0.0.2',
            'new_end_ip': '127.0.0.3'
        })
        self.cmd('dls account firewall create -g {rg} -n {dls} --firewall-rule-name {fw} --start-ip-address {start_ip} --end-ip-address {end_ip}')
        self.cmd('dls account firewall show -g {rg} -n {dls} --firewall-rule-name {fw}', checks=[
            self.check('name', '{fw}'),
            self.check('startIpAddress', '{start_ip}'),
            self.check('endIpAddress', '{end_ip}'),
        ])

        self.cmd('dls account firewall update -g {rg} -n {dls} --firewall-rule-name {fw} --end-ip-address {new_end_ip}')
        self.cmd('dls account firewall show -g {rg} -n {dls} --firewall-rule-name {fw}', checks=[
            self.check('name', '{fw}'),
            self.check('startIpAddress', '{start_ip}'),
            self.check('endIpAddress', '{new_end_ip}'),
        ])

        self.cmd('dls account firewall list -g {rg} -n {dls}', checks=[
            self.check('type(@)', 'array'),
            self.check('length(@)', 1),
        ])
        self.cmd('dls account firewall delete -g {rg} -n {dls} --firewall-rule-name {fw}')
        self.cmd('dls account firewall list -g {rg} -n {dls}', checks=[
            self.check('type(@)', 'array'),
            self.check('length(@)', 0),
        ])

        # test virtual network rule crud
        subnet_id = self.cmd('network vnet subnet show -g {rg} -n default --vnet-name {vnet}').get_output_in_json()['id']
        updated_subnet_id = self.cmd('network vnet subnet create --resource-group {rg} --vnet-name {vnet} --name {updated_subnet} --address-prefixes 10.0.1.0/24').get_output_in_json()['id']

        self.kwargs.update({
            'subnet_id': subnet_id,
            'updated_subnet_id': updated_subnet_id,
            'network_rule': 'dlsVnetRule'
        })
        self.cmd('network vnet subnet update --service-endpoints Microsoft.AzureActiveDirectory --ids "{subnet_id}"')
        self.cmd('network vnet subnet update --service-endpoints Microsoft.AzureActiveDirectory --ids "{updated_subnet_id}"')

        self.cmd('dls account network-rule create -g {rg} --account-name {dls} --name {network_rule} --subnet {subnet_id}')
        self.cmd('dls account network-rule show -g {rg} --account-name {dls} --name {network_rule}', checks=[
            self.check('name', '{network_rule}'),
        ])

        self.cmd('dls account network-rule update -g {rg} --account-name {dls} --name {network_rule} --subnet {updated_subnet_id}')
        self.cmd('dls account network-rule show -g {rg} --account-name {dls} --name {network_rule}', checks=[
            self.check('name', '{network_rule}'),
        ])

        self.cmd('dls account network-rule list -g {rg} --account-name {dls}', checks=[
            self.check('type(@)', 'array'),
            self.check('length(@)', 1),
        ])
        self.cmd('dls account network-rule delete -g {rg} --account-name {dls} --name {network_rule}')
        self.cmd('dls account network-rule list -g {rg} --account-name {dls}', checks=[
            self.check('type(@)', 'array'),
            self.check('length(@)', 0),
        ])
        # test trusted id provider CRUD
        self.kwargs.update({
            'trusted_provider': 'https://sts.windows.net/9d5b43a0-804c-4c82-8791-36aca2f72342',
            'new_provider': 'https://sts.windows.net/fceb709f-96f1-4c65-b06f-2541114bffb3',
            'provider': 'testprovider01'
        })
        self.cmd('dls account trusted-provider create -g {rg} -n {dls} --trusted-id-provider-name {provider} --id-provider {trusted_provider}')
        self.cmd('dls account trusted-provider show -g {rg} -n {dls} --trusted-id-provider-name {provider}', checks=[
            self.check('name', '{provider}'),
            self.check('idProvider', '{trusted_provider}'),
        ])

        self.cmd('dls account trusted-provider update -g {rg} -n {dls} --trusted-id-provider-name {provider} --id-provider {new_provider}')
        self.cmd('dls account trusted-provider show -g {rg} -n {dls} --trusted-id-provider-name {provider}', checks=[
            self.check('name', '{provider}'),
            self.check('idProvider', '{new_provider}'),
        ])

        self.cmd('dls account trusted-provider list -g {rg} -n {dls}', checks=[
            self.check('type(@)', 'array'),
            self.check('length(@)', 1),
        ])
        self.cmd('dls account trusted-provider delete -g {rg} -n {dls} --trusted-id-provider-name {provider}')
        self.cmd('dls account trusted-provider list -g {rg} -n {dls}', checks=[
            self.check('type(@)', 'array'),
            self.check('length(@)', 0),
        ])

        # test account deletion
        self.cmd('dls account delete -g {rg} -n {dls}')
        self.cmd('dls account list -g {rg}', checks=[
            self.check('type(@)', 'array'),
            self.check('length(@)', 0),
        ])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# AZURE CLI EventHub - NAMESPACE TEST DEFINITIONS

import time

from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer)


# pylint: disable=line-too-long
# pylint: disable=too-many-lines


class EHEventhubCURDScenarioTest(ScenarioTest):

    @ResourceGroupPreparer(name_prefix='cli_test_eh_eventnhub')
    def test_eh_eventhub(self, resource_group):
        self.kwargs.update({
            'loc': 'westus2',
            'rg': resource_group,
            'namespacename': self.create_random_name(prefix='eventhubs-nscli', length=20),
            'tags': {'tag1: value1', 'tag2: value2'},
            'sku': 'Standard',
            'tier': 'Standard',
            'authoname': self.create_random_name(prefix='cliAutho', length=20),
            'defaultauthorizationrule': 'RootManageSharedAccessKey',
            'accessrights': 'Listen',
            'accessrights1': 'Send',
            'primary': 'PrimaryKey',
            'secondary': 'SecondaryKey',
            'eventhubname': self.create_random_name(prefix='eventhubs-eventhubcli', length=25),
            'eventhubauthoname': self.create_random_name(prefix='cliEventAutho', length=25),
            'isautoinflateenabled': 'True',
            'maximumthroughputunits': 4,
            'messageretentionindays': 4,
            'partitioncount': 4,
            'destinationname': 'EventHubArchive.AzureBlockBlob',
            'storageaccount': '',
            'blobcontainer': 'container01',
            'archinvenameformat': '{Namespace}/{EventHub}/{PartitionId}/{Year}/{Month}/{Day}/{Hour}/{Minute}/{Second}'
        })

        # updated teh Storageaccount ID
        subid = self.cmd('account show --query id -otsv').output.replace('\n', '')
        storageaccountid = '/subscriptions/' + subid + '/resourcegroups/v-ajnavtestcli/providers/Microsoft.Storage/storageAccounts/testing1sdkeventhub'
        self.kwargs.update({'storageaccount': storageaccountid})

        # Create Namespace
        self.cmd(
            'eventhubs namespace create --resource-group {rg} --name {namespacename} --location {loc} --tags {tags} --sku {sku} --enable-auto-inflate {isautoinflateenabled} --maximum-throughput-units {maximumthroughputunits}',
            checks=[self.check('sku.name', self.kwargs['sku'])])

        # Get Created Namespace
        self.cmd('eventhubs namespace show --resource-group {rg} --name {namespacename}',
                 checks=[self.check('sku.name', self.kwargs['sku'])])

        # Create Eventhub
        self.cmd(
            'eventhubs eventhub create --resource-group {rg} --namespace-name {namespacename} --name {eventhubname}',
            checks=[self.check('name', self.kwargs['eventhubname'])])

        # Get Eventhub
        self.cmd('eventhubs eventhub show --resource-group {rg} --namespace-name {namespacename} --name {eventhubname}',
                 checks=[self.check('name', self.kwargs['eventhubname'])])

        # update Eventhub
        self.cmd(
            'eventhubs eventhub update --resource-group {rg} --namespace-name {namespacename} --name {eventhubname} --partition-count {partitioncount} --message-retention {messageretentionindays}',
            checks=[self.check('name', self.kwargs['eventhubname'])])

        # update Eventhub
        self.cmd(
            'eventhubs eventhub update --resource-group {rg} --namespace-name {namespacename} --name {eventhubname} --enable-capture {isautoinflateenabled} --skip-empty-archives {isautoinflateenabled} --capture-interval 120 --capture-size-limit 10485763 --destination-name {destinationname} --storage-account {storageaccount} --blob-container {blobcontainer} --archive-name-format {archinvenameformat} ',
            checks=[self.check('name', self.kwargs['eventhubname'])])

        # Eventhub List
        listeventhub = self.cmd('eventhubs eventhub list --resource-group {rg} --namespace-name {namespacename}').output
        self.assertGreater(len(listeventhub), 0)

        # Create Authoriazation Rule
        self.cmd(
            'eventhubs eventhub authorization-rule create --resource-group {rg} --namespace-name {namespacename} --eventhub-name {eventhubname} --name {authoname} --rights {accessrights}',
            checks=[self.check('name', self.kwargs['authoname'])])

        # Get Create Authorization Rule
        self.cmd(
            'eventhubs eventhub authorization-rule show --resource-group {rg} --namespace-name {namespacename} --eventhub-name {eventhubname} --name {authoname}',
            checks=[self.check('name', self.kwargs['authoname'])])

        # update Authoriazation Rule
        self.cmd(
            'eventhubs eventhub authorization-rule update --resource-group {rg} --namespace-name {namespacename} --eventhub-name {eventhubname} --name {authoname} --rights {accessrights1}',
            checks=[self.check('name', self.kwargs['authoname'])])

        # Get Authorization Rule Listkeys
        self.cmd(
            'eventhubs eventhub authorization-rule keys list --resource-group {rg} --namespace-name {namespacename} --eventhub-name {eventhubname} --name {authoname}')

        # Regeneratekeys - Primary
        regenrateprimarykeyresult = self.cmd(
            'eventhubs eventhub authorization-rule keys renew --resource-group {rg} --namespace-name {namespacename} --eventhub-name {eventhubname} --name {authoname} --key {primary}')
        self.assertIsNotNone(regenrateprimarykeyresult)

        # Regeneratekeys - Secondary
        regenratesecondarykeyresult = self.cmd(
            'eventhubs eventhub authorization-rule keys renew --resource-group {rg} --namespace-name {namespacename} --eventhub-name {eventhubname} --name {authoname} --key {secondary}')
        self.assertIsNotNone(regenratesecondarykeyresult)

        # Delete Eventhub AuthorizationRule
        self.cmd(
            'eventhubs eventhub authorization-rule delete --resource-group {rg} --namespace-name {namespacename} --eventhub-name {eventhubname} --name {authoname}')

        # Delete Eventhub
        self.cmd(
            'eventhubs eventhub delete --resource-group {rg} --namespace-name {namespacename} --name {eventhubname}')

        # Delete Namespace
        self.cmd('eventhubs namespace delete --resource-group {rg} --name {namespacename}')
